# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..types import navigation_retrieve_route_params
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.navigation_retrieve_route_response import NavigationRetrieveRouteResponse

__all__ = ["NavigationResource", "AsyncNavigationResource"]


class NavigationResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> NavigationResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#accessing-raw-response-data-eg-headers
        """
        return NavigationResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> NavigationResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#with_streaming_response
        """
        return NavigationResourceWithStreamingResponse(self)

    def retrieve_route(
        self,
        *,
        key: str,
        altcount: int | NotGiven = NOT_GIVEN,
        alternatives: bool | NotGiven = NOT_GIVEN,
        approaches: Literal["`unrestricted`", "`curb`"] | NotGiven = NOT_GIVEN,
        avoid: Literal["toll", "ferry", "highway", "none"] | NotGiven = NOT_GIVEN,
        bearings: str | NotGiven = NOT_GIVEN,
        destination: str | NotGiven = NOT_GIVEN,
        geometry: Literal["`polyline`", "`polyline6`", "`geojson`"] | NotGiven = NOT_GIVEN,
        lang: str | NotGiven = NOT_GIVEN,
        mode: Literal["`car`", "`truck`"] | NotGiven = NOT_GIVEN,
        origin: str | NotGiven = NOT_GIVEN,
        original_shape: str | NotGiven = NOT_GIVEN,
        original_shape_type: Literal["polyline", "polyline6"] | NotGiven = NOT_GIVEN,
        overview: Literal["`full`", "`simplified`", "`false`"] | NotGiven = NOT_GIVEN,
        waypoints: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> NavigationRetrieveRouteResponse:
        """
        Nextbillion.ai’s Navigation API is a service that computes a route between 2
        places, and also returns detailed turn by turn instructions for the route.

        The Navigation API can be used as an input into your Navigation app.
        Alternatively, you can directly use Nextbillion.ai’s Navigation SDK for a
        complete turn by turn navigation experience.

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          altcount: Sets the number of alternative routes to return. It is effective only when
              "alternatives" is "true". Please note that adding alternative route count does
              not guarantee matching number of routes to be returned if potential alternative
              routes do not exist.

          alternatives: When "true" the API will return alternate routes. The "alternatives" is
              effective only when there are no "waypoints" included in the request. You can
              set the number of alternate routes to be returned in the "altcount" property.

          approaches: A semicolon-separated list indicating the side of the road from which to
              approach "waypoints" in a requested route. When set to "unrestricted" a route
              can arrive at the waypoint from either side of the road and when set to "curb"
              the route will arrive at the waypoint on the driving side of the region. Please
              note the number of values provided must be one more than the number of
              "waypoints". The last value of "approaches" will determine the approach for the
              "destination". However, you can skip a coordinate and show its position in the
              list with the ";" separator.

          avoid: Setting this will ensure the route avoids ferries, tolls, highways or nothing.
              Multiple values should be separated by a pipe (|). If "none" is provided along
              with other values, an error is returned as a valid route is not feasible. Please
              note that when this parameter is not provided in the input, ferries are set to
              be avoided by default. When this parameter is provided, only the mentioned
              objects are avoided.

          bearings: Limits the search to road segments with given bearing, in degrees, towards true
              north in clockwise direction. Each "bearings" should be in the format of
              "degree,range", where the "degree" should be a value between \\[[0, 360\\]] and
              "range" should be a value between \\[[0, 180\\]]. Please note that the number of
              "bearings" should be two more than the number of "waypoints". This is to account
              for the bearing of "origin" and "destination". If a route can approach a
              "waypoint" or the "destination" from any direction, the bearing for that point
              can be specified as "0,180".

          destination: "destination" is the ending point of your route. Ensure that the "destination"
              is a routable land location. Please note that this parameter is mandatory if the
              "original_shape" parameter is not given.

          geometry: Sets the output format of the route geometry in the response. On providing
              “polyline“ and “polyline6“ as input, respective encoded geometry is returned.
              However, when “geojson“ is provided as the input value, “polyline“ encoded
              geometry is returned in the response along with the geojson details of the
              route.

          lang: Select the language to be used for result rendering from a list of
              [BCP 47](https://en.wikipedia.org/wiki/IETF_language_tag) compliant language
              codes.

          mode: Set which driving mode the service should use to determine a route. For example,
              if you use "car", the API will return a route that a car can take. Using "truck"
              will return a route a truck can use, taking into account appropriate truck
              routing restrictions.

              When "mode=truck", following are the default dimensions that are used:

              \\-- truck_height = 214 centimeters

              \\-- truck_width = 183 centimeters

              \\-- truck_length = 519 centimeters

              \\-- truck_weight = 5000 kg

              Please use the Navigation Flexible version if you want to use custom truck
              dimensions.

              Note: Only the "car" profile is enabled by default. Please note that customized
              profiles (including "truck") might not be available for all regions. Please
              contact your [NextBillion.ai](http://NextBillion.ai) account manager, sales
              representative or reach out at
              [support@nextbillion.ai](mailto:support@nextbillion.ai) in case you need
              additional profiles.

          origin: "origin" is the starting point of your route. Ensure that "origin" is a routable
              land location. Please note that this parameter is mandatory if the geometry
              parameter is not given.

          original_shape: Takes a route geometry as input and returns the navigation information for that
              route. Accepts "polyline" and "polyline6" encoded geometry as input.
              "original_shape_type" becomes mandatory when "original_shape" is used. If this
              parameter is provided, the only other parameters which will be considered are
              "original_shape_type", "lang", "geometry". The rest of the parameters in the
              input request will be ignored. Please note overview verbosity will always be
              "full" when using this parameter.

          original_shape_type: Specify the encoding format of route geometry provided in the request using
              "original_shape" parameter. Please note that an error is returned when this
              parameter is not specified while an input is added to "original_shape"
              parameter.

          overview: Specify the verbosity of route geometry. When set to "full", the most detailed
              geometry available is returned. When set to "simplified", a simplified version
              of the full geometry is returned. No overview geometry is returned when set to
              "false".

          waypoints: "waypoints" are coordinates along the route between the "origin" and
              "destination". It is a pipe-separated list of coordinate pairs. Please note that
              the route returned will arrive at the "waypoints" in the sequence they are
              provided in the input request. Please note that the maximum number of waypoints
              that can be provided in a single request is 50 when using GET method and 200
              with POST method.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/navigation/json",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "key": key,
                        "altcount": altcount,
                        "alternatives": alternatives,
                        "approaches": approaches,
                        "avoid": avoid,
                        "bearings": bearings,
                        "destination": destination,
                        "geometry": geometry,
                        "lang": lang,
                        "mode": mode,
                        "origin": origin,
                        "original_shape": original_shape,
                        "original_shape_type": original_shape_type,
                        "overview": overview,
                        "waypoints": waypoints,
                    },
                    navigation_retrieve_route_params.NavigationRetrieveRouteParams,
                ),
            ),
            cast_to=NavigationRetrieveRouteResponse,
        )


class AsyncNavigationResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncNavigationResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncNavigationResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncNavigationResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#with_streaming_response
        """
        return AsyncNavigationResourceWithStreamingResponse(self)

    async def retrieve_route(
        self,
        *,
        key: str,
        altcount: int | NotGiven = NOT_GIVEN,
        alternatives: bool | NotGiven = NOT_GIVEN,
        approaches: Literal["`unrestricted`", "`curb`"] | NotGiven = NOT_GIVEN,
        avoid: Literal["toll", "ferry", "highway", "none"] | NotGiven = NOT_GIVEN,
        bearings: str | NotGiven = NOT_GIVEN,
        destination: str | NotGiven = NOT_GIVEN,
        geometry: Literal["`polyline`", "`polyline6`", "`geojson`"] | NotGiven = NOT_GIVEN,
        lang: str | NotGiven = NOT_GIVEN,
        mode: Literal["`car`", "`truck`"] | NotGiven = NOT_GIVEN,
        origin: str | NotGiven = NOT_GIVEN,
        original_shape: str | NotGiven = NOT_GIVEN,
        original_shape_type: Literal["polyline", "polyline6"] | NotGiven = NOT_GIVEN,
        overview: Literal["`full`", "`simplified`", "`false`"] | NotGiven = NOT_GIVEN,
        waypoints: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> NavigationRetrieveRouteResponse:
        """
        Nextbillion.ai’s Navigation API is a service that computes a route between 2
        places, and also returns detailed turn by turn instructions for the route.

        The Navigation API can be used as an input into your Navigation app.
        Alternatively, you can directly use Nextbillion.ai’s Navigation SDK for a
        complete turn by turn navigation experience.

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          altcount: Sets the number of alternative routes to return. It is effective only when
              "alternatives" is "true". Please note that adding alternative route count does
              not guarantee matching number of routes to be returned if potential alternative
              routes do not exist.

          alternatives: When "true" the API will return alternate routes. The "alternatives" is
              effective only when there are no "waypoints" included in the request. You can
              set the number of alternate routes to be returned in the "altcount" property.

          approaches: A semicolon-separated list indicating the side of the road from which to
              approach "waypoints" in a requested route. When set to "unrestricted" a route
              can arrive at the waypoint from either side of the road and when set to "curb"
              the route will arrive at the waypoint on the driving side of the region. Please
              note the number of values provided must be one more than the number of
              "waypoints". The last value of "approaches" will determine the approach for the
              "destination". However, you can skip a coordinate and show its position in the
              list with the ";" separator.

          avoid: Setting this will ensure the route avoids ferries, tolls, highways or nothing.
              Multiple values should be separated by a pipe (|). If "none" is provided along
              with other values, an error is returned as a valid route is not feasible. Please
              note that when this parameter is not provided in the input, ferries are set to
              be avoided by default. When this parameter is provided, only the mentioned
              objects are avoided.

          bearings: Limits the search to road segments with given bearing, in degrees, towards true
              north in clockwise direction. Each "bearings" should be in the format of
              "degree,range", where the "degree" should be a value between \\[[0, 360\\]] and
              "range" should be a value between \\[[0, 180\\]]. Please note that the number of
              "bearings" should be two more than the number of "waypoints". This is to account
              for the bearing of "origin" and "destination". If a route can approach a
              "waypoint" or the "destination" from any direction, the bearing for that point
              can be specified as "0,180".

          destination: "destination" is the ending point of your route. Ensure that the "destination"
              is a routable land location. Please note that this parameter is mandatory if the
              "original_shape" parameter is not given.

          geometry: Sets the output format of the route geometry in the response. On providing
              “polyline“ and “polyline6“ as input, respective encoded geometry is returned.
              However, when “geojson“ is provided as the input value, “polyline“ encoded
              geometry is returned in the response along with the geojson details of the
              route.

          lang: Select the language to be used for result rendering from a list of
              [BCP 47](https://en.wikipedia.org/wiki/IETF_language_tag) compliant language
              codes.

          mode: Set which driving mode the service should use to determine a route. For example,
              if you use "car", the API will return a route that a car can take. Using "truck"
              will return a route a truck can use, taking into account appropriate truck
              routing restrictions.

              When "mode=truck", following are the default dimensions that are used:

              \\-- truck_height = 214 centimeters

              \\-- truck_width = 183 centimeters

              \\-- truck_length = 519 centimeters

              \\-- truck_weight = 5000 kg

              Please use the Navigation Flexible version if you want to use custom truck
              dimensions.

              Note: Only the "car" profile is enabled by default. Please note that customized
              profiles (including "truck") might not be available for all regions. Please
              contact your [NextBillion.ai](http://NextBillion.ai) account manager, sales
              representative or reach out at
              [support@nextbillion.ai](mailto:support@nextbillion.ai) in case you need
              additional profiles.

          origin: "origin" is the starting point of your route. Ensure that "origin" is a routable
              land location. Please note that this parameter is mandatory if the geometry
              parameter is not given.

          original_shape: Takes a route geometry as input and returns the navigation information for that
              route. Accepts "polyline" and "polyline6" encoded geometry as input.
              "original_shape_type" becomes mandatory when "original_shape" is used. If this
              parameter is provided, the only other parameters which will be considered are
              "original_shape_type", "lang", "geometry". The rest of the parameters in the
              input request will be ignored. Please note overview verbosity will always be
              "full" when using this parameter.

          original_shape_type: Specify the encoding format of route geometry provided in the request using
              "original_shape" parameter. Please note that an error is returned when this
              parameter is not specified while an input is added to "original_shape"
              parameter.

          overview: Specify the verbosity of route geometry. When set to "full", the most detailed
              geometry available is returned. When set to "simplified", a simplified version
              of the full geometry is returned. No overview geometry is returned when set to
              "false".

          waypoints: "waypoints" are coordinates along the route between the "origin" and
              "destination". It is a pipe-separated list of coordinate pairs. Please note that
              the route returned will arrive at the "waypoints" in the sequence they are
              provided in the input request. Please note that the maximum number of waypoints
              that can be provided in a single request is 50 when using GET method and 200
              with POST method.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/navigation/json",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "key": key,
                        "altcount": altcount,
                        "alternatives": alternatives,
                        "approaches": approaches,
                        "avoid": avoid,
                        "bearings": bearings,
                        "destination": destination,
                        "geometry": geometry,
                        "lang": lang,
                        "mode": mode,
                        "origin": origin,
                        "original_shape": original_shape,
                        "original_shape_type": original_shape_type,
                        "overview": overview,
                        "waypoints": waypoints,
                    },
                    navigation_retrieve_route_params.NavigationRetrieveRouteParams,
                ),
            ),
            cast_to=NavigationRetrieveRouteResponse,
        )


class NavigationResourceWithRawResponse:
    def __init__(self, navigation: NavigationResource) -> None:
        self._navigation = navigation

        self.retrieve_route = to_raw_response_wrapper(
            navigation.retrieve_route,
        )


class AsyncNavigationResourceWithRawResponse:
    def __init__(self, navigation: AsyncNavigationResource) -> None:
        self._navigation = navigation

        self.retrieve_route = async_to_raw_response_wrapper(
            navigation.retrieve_route,
        )


class NavigationResourceWithStreamingResponse:
    def __init__(self, navigation: NavigationResource) -> None:
        self._navigation = navigation

        self.retrieve_route = to_streamed_response_wrapper(
            navigation.retrieve_route,
        )


class AsyncNavigationResourceWithStreamingResponse:
    def __init__(self, navigation: AsyncNavigationResource) -> None:
        self._navigation = navigation

        self.retrieve_route = async_to_streamed_response_wrapper(
            navigation.retrieve_route,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from tests.utils import assert_matches_type
from nextbillion_sdk import NextbillionSDK, AsyncNextbillionSDK
from nextbillion_sdk.types.skynet.asset import (
    LocationListResponse,
    LocationGetLastResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestLocation:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    def test_method_list(self, client: NextbillionSDK) -> None:
        location = client.skynet.asset.location.list(
            id="id",
            key="key=API_KEY",
        )
        assert_matches_type(LocationListResponse, location, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_list_with_all_params(self, client: NextbillionSDK) -> None:
        location = client.skynet.asset.location.list(
            id="id",
            key="key=API_KEY",
            cluster="america",
            correction="correction=mapmatch=1,interpolate=0,mode=`car`",
            end_time=0,
            geometry_type="`polyline`",
            pn=0,
            ps=500,
            start_time=0,
        )
        assert_matches_type(LocationListResponse, location, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_list(self, client: NextbillionSDK) -> None:
        response = client.skynet.asset.location.with_raw_response.list(
            id="id",
            key="key=API_KEY",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        location = response.parse()
        assert_matches_type(LocationListResponse, location, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_list(self, client: NextbillionSDK) -> None:
        with client.skynet.asset.location.with_streaming_response.list(
            id="id",
            key="key=API_KEY",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            location = response.parse()
            assert_matches_type(LocationListResponse, location, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_list(self, client: NextbillionSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.skynet.asset.location.with_raw_response.list(
                id="",
                key="key=API_KEY",
            )

    @pytest.mark.skip()
    @parametrize
    def test_method_get_last(self, client: NextbillionSDK) -> None:
        location = client.skynet.asset.location.get_last(
            id="id",
            key="key=API_KEY",
        )
        assert_matches_type(LocationGetLastResponse, location, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_get_last_with_all_params(self, client: NextbillionSDK) -> None:
        location = client.skynet.asset.location.get_last(
            id="id",
            key="key=API_KEY",
            cluster="america",
        )
        assert_matches_type(LocationGetLastResponse, location, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_get_last(self, client: NextbillionSDK) -> None:
        response = client.skynet.asset.location.with_raw_response.get_last(
            id="id",
            key="key=API_KEY",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        location = response.parse()
        assert_matches_type(LocationGetLastResponse, location, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_get_last(self, client: NextbillionSDK) -> None:
        with client.skynet.asset.location.with_streaming_response.get_last(
            id="id",
            key="key=API_KEY",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            location = response.parse()
            assert_matches_type(LocationGetLastResponse, location, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_get_last(self, client: NextbillionSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.skynet.asset.location.with_raw_response.get_last(
                id="",
                key="key=API_KEY",
            )


class TestAsyncLocation:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip()
    @parametrize
    async def test_method_list(self, async_client: AsyncNextbillionSDK) -> None:
        location = await async_client.skynet.asset.location.list(
            id="id",
            key="key=API_KEY",
        )
        assert_matches_type(LocationListResponse, location, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncNextbillionSDK) -> None:
        location = await async_client.skynet.asset.location.list(
            id="id",
            key="key=API_KEY",
            cluster="america",
            correction="correction=mapmatch=1,interpolate=0,mode=`car`",
            end_time=0,
            geometry_type="`polyline`",
            pn=0,
            ps=500,
            start_time=0,
        )
        assert_matches_type(LocationListResponse, location, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncNextbillionSDK) -> None:
        response = await async_client.skynet.asset.location.with_raw_response.list(
            id="id",
            key="key=API_KEY",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        location = await response.parse()
        assert_matches_type(LocationListResponse, location, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncNextbillionSDK) -> None:
        async with async_client.skynet.asset.location.with_streaming_response.list(
            id="id",
            key="key=API_KEY",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            location = await response.parse()
            assert_matches_type(LocationListResponse, location, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_list(self, async_client: AsyncNextbillionSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.skynet.asset.location.with_raw_response.list(
                id="",
                key="key=API_KEY",
            )

    @pytest.mark.skip()
    @parametrize
    async def test_method_get_last(self, async_client: AsyncNextbillionSDK) -> None:
        location = await async_client.skynet.asset.location.get_last(
            id="id",
            key="key=API_KEY",
        )
        assert_matches_type(LocationGetLastResponse, location, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_get_last_with_all_params(self, async_client: AsyncNextbillionSDK) -> None:
        location = await async_client.skynet.asset.location.get_last(
            id="id",
            key="key=API_KEY",
            cluster="america",
        )
        assert_matches_type(LocationGetLastResponse, location, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_get_last(self, async_client: AsyncNextbillionSDK) -> None:
        response = await async_client.skynet.asset.location.with_raw_response.get_last(
            id="id",
            key="key=API_KEY",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        location = await response.parse()
        assert_matches_type(LocationGetLastResponse, location, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_get_last(self, async_client: AsyncNextbillionSDK) -> None:
        async with async_client.skynet.asset.location.with_streaming_response.get_last(
            id="id",
            key="key=API_KEY",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            location = await response.parse()
            assert_matches_type(LocationGetLastResponse, location, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_get_last(self, async_client: AsyncNextbillionSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.skynet.asset.location.with_raw_response.get_last(
                id="",
                key="key=API_KEY",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = [
    "NavigationRetrieveRouteResponse",
    "Route",
    "RouteEndLocation",
    "RouteGeojson",
    "RouteLeg",
    "RouteLegDistance",
    "RouteLegDuration",
    "RouteLegEndLocation",
    "RouteLegStartLocation",
    "RouteLegStep",
    "RouteLegStepDistance",
    "RouteLegStepDuration",
    "RouteLegStepEndLocation",
    "RouteLegStepGeojson",
    "RouteLegStepIntersection",
    "RouteLegStepIntersectionLane",
    "RouteLegStepIntersectionLocation",
    "RouteLegStepManeuver",
    "RouteLegStepManeuverCoordinate",
    "RouteLegStepManeuverVoiceInstruction",
    "RouteLegStepRoadShieldType",
    "RouteLegStepStartLocation",
    "RouteStartLocation",
]


class RouteEndLocation(BaseModel):
    latitude: Optional[float] = None
    """Latitude of the `end_location`."""

    longitude: Optional[float] = None
    """Longitude of the `end_location`."""


class RouteGeojson(BaseModel):
    geometry: Optional[str] = None

    properties: Optional[str] = None

    type: Optional[
        Literal[
            "Point",
            "MultiPoint",
            "LineString",
            "MultiLineString",
            "Polygon",
            "MultiPolygon",
            "GeometryCollection",
            "Feature",
            "FeatureCollection",
            "Link",
        ]
    ] = None


class RouteLegDistance(BaseModel):
    value: Optional[int] = None


class RouteLegDuration(BaseModel):
    value: Optional[int] = None


class RouteLegEndLocation(BaseModel):
    latitude: Optional[float] = None
    """Latitude of `end_location` of the `leg`."""

    longitude: Optional[float] = None
    """Longitude of `end_location` of the `leg`."""


class RouteLegStartLocation(BaseModel):
    latitude: Optional[float] = None
    """Latitude of `start_location` of the `leg`."""

    longitude: Optional[float] = None
    """Longitude of `start_location` of the `leg`."""


class RouteLegStepDistance(BaseModel):
    value: Optional[int] = None


class RouteLegStepDuration(BaseModel):
    value: Optional[int] = None


class RouteLegStepEndLocation(BaseModel):
    latitude: Optional[float] = None
    """Latitude of the `end_location` of the `step`."""

    longitude: Optional[float] = None
    """Longitude of the `end_location` of the `step`."""


class RouteLegStepGeojson(BaseModel):
    geometry: Optional[str] = None

    type: Optional[str] = None


class RouteLegStepIntersectionLane(BaseModel):
    indications: Optional[List[str]] = None
    """It represents actions associated with the lane.

    These indications describe the permitted maneuvers or directions that can be
    taken from the lane. Common indications include "turn left," "turn right," "go
    straight," "merge," "exit," etc.
    """

    valid: Optional[bool] = None
    """This indicates the validity of the lane.

    It specifies whether the lane is considered valid for making the indicated
    maneuver or if there are any restrictions or limitations associated with it.
    """


class RouteLegStepIntersectionLocation(BaseModel):
    latitude: Optional[float] = None
    """The latitude coordinate of the `intersection`."""

    longitude: Optional[float] = None
    """The longitude coordinate of the `intersection`."""

    name: Optional[str] = None
    """The name or description of the `intersection`."""


class RouteLegStepIntersection(BaseModel):
    bearings: Optional[List[int]] = None
    """A list of bearing values (e.g.

    [0,90,180,270]) that are available at the intersection. The `bearings` describe
    all available roads at the intersection.
    """

    classes: Optional[List[str]] = None
    """
    An array of strings representing the classes or types of roads or paths at the
    intersection. The classes can indicate the road hierarchy, such as a motorway,
    primary road, secondary road, etc.
    """

    entry: Optional[List[bool]] = None
    """
    A value of `true` indicates that the respective road could be entered on a valid
    route. `false` indicates that the turn onto the respective road would violate a
    restriction. Each entry value corresponds to the bearing angle at the same
    index.
    """

    intersection_in: Optional[int] = None
    """The number of incoming roads or paths at the `intersection`."""

    intersection_out: Optional[int] = None
    """The number of outgoing roads or paths from the `intersection`."""

    lanes: Optional[List[RouteLegStepIntersectionLane]] = None
    """An array of lane objects representing the lanes available at the intersection.

    If no lane information is available for an `intersection`, the `lanes` property
    will not be present.
    """

    location: Optional[RouteLegStepIntersectionLocation] = None
    """A [longitude, latitude] pair describing the location of the intersection."""


class RouteLegStepManeuverCoordinate(BaseModel):
    latitude: Optional[float] = None
    """The latitude coordinate of the `maneuver`."""

    longitude: Optional[float] = None
    """The longitude coordinate of the `maneuver`."""

    name: Optional[str] = None
    """The name or description of the maneuver location."""


class RouteLegStepManeuverVoiceInstruction(BaseModel):
    distance_along_geometry: Optional[int] = None

    instruction: Optional[str] = None
    """The guidance instructions for the upcoming `maneuver`"""

    unit: Optional[str] = None
    """Unit of the `distance_along_geometry` metric"""


class RouteLegStepManeuver(BaseModel):
    bearing_after: Optional[float] = None
    """
    The clockwise angle from true north to the direction of travel immediately after
    the `maneuver`. Range of values is between 0-359.
    """

    bearing_before: Optional[float] = None
    """
    The clockwise angle from true north to the direction of travel immediately
    before the `maneuver`. Range of values is between 0-359.
    """

    coordinate: Optional[RouteLegStepManeuverCoordinate] = None
    """A coordinate pair describing the location of the `maneuver`."""

    instruction: Optional[str] = None
    """A text instruction describing the maneuver to be performed.

    It provides guidance on the action to take at the maneuver location, such as
    "Turn left," "Go straight," "Exit the roundabout," etc.
    """

    maneuver_type: Optional[str] = None
    """A string indicating the type of `maneuver`."""

    muted: Optional[bool] = None
    """
    A boolean value indicating whether the voice instruction for the maneuver should
    be muted or not.
    """

    roundabout_count: Optional[int] = None
    """The number of roundabouts encountered so far during the route.

    This parameter is specific to roundabout maneuvers and indicates the count of
    roundabouts before the current one.
    """

    voice_instruction: Optional[List[RouteLegStepManeuverVoiceInstruction]] = None
    """An array of voice instruction objects associated with the `maneuver`.

    Each object provides additional details about the voice instruction, including
    the distance along the geometry where the instruction applies, the instruction
    text, and the unit of measurement.
    """


class RouteLegStepRoadShieldType(BaseModel):
    image_url: Optional[str] = None
    """The URL to fetch the road shield image."""

    label: Optional[str] = None
    """
    A label identifying the inscription on the road shield, such as containing the
    road number.
    """


class RouteLegStepStartLocation(BaseModel):
    latitude: Optional[float] = None
    """Latitude of `start_location` of the `step`."""

    longitude: Optional[float] = None
    """Longitude of `start_location` of the `step`."""


class RouteLegStep(BaseModel):
    distance: Optional[RouteLegStepDistance] = None
    """An object containing step distance value, in meters."""

    driving_side: Optional[str] = None
    """
    Indicates the driving side of the road in case bidirectional traffic is allowed
    on the given segment. It can have two values: "left" & "right".
    """

    duration: Optional[RouteLegStepDuration] = None
    """An object containing step duration value, in seconds."""

    end_location: Optional[RouteLegStepEndLocation] = None
    """Location coordinates of the point where the `step` ends."""

    geojson: Optional[RouteLegStepGeojson] = None
    """The GeoJSON representation of the `step`."""

    geometry: Optional[str] = None
    """Encoded geometry of the `step` in the selected format."""

    intersections: Optional[List[RouteLegStepIntersection]] = None
    """
    An array of objects representing intersections (or cross-way) that the route
    passes by along the `step`. For every `step`, the very first `intersection`
    corresponds to the location of the `maneuver`. All intersections until the next
    `maneuver` are listed in this object.
    """

    maneuver: Optional[RouteLegStepManeuver] = None
    """An object with maneuver details for the `step`."""

    name: Optional[str] = None
    """The name of the `step`."""

    reference: Optional[str] = None
    """A reference for the `step`."""

    road_shield_type: Optional[RouteLegStepRoadShieldType] = None
    """An object containing road shield information."""

    start_location: Optional[RouteLegStepStartLocation] = None
    """Location coordinates of the point where the `step` starts."""


class RouteLeg(BaseModel):
    distance: Optional[RouteLegDistance] = None
    """An object containing leg distance value, in meters."""

    duration: Optional[RouteLegDuration] = None
    """An object containing leg duration value, in seconds."""

    end_location: Optional[RouteLegEndLocation] = None
    """Location coordinates of the point where the leg ends."""

    raw_duration: Optional[object] = None
    """The raw estimated duration of the `leg` in seconds."""

    start_location: Optional[RouteLegStartLocation] = None
    """Location coordinates of the point where the leg starts."""

    steps: Optional[List[RouteLegStep]] = None
    """An array of step objects containing turn-by-turn guidance for easy navigation."""


class RouteStartLocation(BaseModel):
    latitude: Optional[float] = None
    """Latitude of the`start_location`."""

    longitude: Optional[float] = None
    """Longitude of the `start_location`."""


class Route(BaseModel):
    distance: Optional[float] = None
    """The distance, in meters, of the complete trip."""

    distance_full: Optional[float] = None
    """
    The total distance of the route, including additional details such as extra
    maneuvers or loops, in meters.
    """

    duration: Optional[int] = None
    """The duration, in seconds, of the complete trip."""

    end_location: Optional[RouteEndLocation] = None
    """Location coordinates of the point where the route ends."""

    geojson: Optional[RouteGeojson] = None
    """The GeoJSON representation of the route."""

    geometry: Optional[str] = None
    """
    Encoded geometry of the returned route as per the selected format in `geometry`
    and specified `overview` verbosity. Please note the `overview` will always be
    `full` when `original_shape` parameter is used in the input request.
    """

    legs: Optional[List[RouteLeg]] = None
    """An array of objects returning the details about each `leg` of the route.

    `waypoints` split the route into legs.
    """

    predicted_duration: Optional[float] = None
    """The predicted duration of the route based on real-time traffic conditions."""

    raw_duration: Optional[float] = None
    """The raw estimated duration of the route in seconds."""

    special_objects: Optional[object] = None
    """Special geospatial objects or landmarks crossed along the route."""

    start_location: Optional[RouteStartLocation] = None
    """Location coordinates of the point where the route starts."""

    weight: Optional[float] = None
    """A weight value associated with the route or leg."""


class NavigationRetrieveRouteResponse(BaseModel):
    msg: Optional[str] = None
    """Displays the error message in case of a failed request or operation.

    Please note that this parameter is not returned in the response in case of a
    successful request.
    """

    routes: Optional[List[Route]] = None
    """
    An array of objects describing different possible routes from the starting
    location to the destination. Each object represents one route.
    """

    status: Optional[str] = None
    """A string indicating the state of the response.

    On normal responses, the value will be `Ok`. Indicative HTTP error codes are
    returned for different errors. See the [API Errors Codes](#api-error-codes)
    section below for more information.
    """

    warning: Optional[List[str]] = None
    """warning when facing unexpected behaviour"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["TripStop"]


class TripStop(BaseModel):
    geofence_id: Optional[str] = None
    """
    Returns the ID of the geofence that was used to indicate the area to make a
    stop.
    """

    meta_data: Optional[object] = None
    """
    Returns any meta data that was added to provide additional information about the
    stop.
    """

    name: Optional[str] = None
    """
    Returns the name of the stop that was provided when configuring this stop for
    the trip.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["DocumentTemplateDeleteResponse"]


class DocumentTemplateDeleteResponse(BaseModel):
    msg: Optional[str] = None
    """Returns the error message in case of a failed request.

    If the request is successful, this field is not present in the response.
    """

    status: Optional[int] = None
    """Returns the HTTP response code."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Required, TypedDict

__all__ = ["BatchDeleteParams"]


class BatchDeleteParams(TypedDict, total=False):
    key: Required[str]
    """
    A key is a unique identifier that is required to authenticate a request to the
    API.
    """

    ids: List[str]
    """An array IDs of the geofence to be deleted.

    These are the IDs that were generated/provided at the time of creating the
    respective geofences.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from tests.utils import assert_matches_type
from nextbillion_sdk import NextbillionSDK, AsyncNextbillionSDK
from nextbillion_sdk.types.fleetify import (
    RouteCreateResponse,
    RouteRedispatchResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestRoutes:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    def test_method_create(self, client: NextbillionSDK) -> None:
        route = client.fleetify.routes.create(
            key="key",
            driver_email="johndoe@abc.com",
            steps=[
                {
                    "arrival": 0,
                    "location": [0],
                    "type": "`start`",
                }
            ],
        )
        assert_matches_type(RouteCreateResponse, route, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_create_with_all_params(self, client: NextbillionSDK) -> None:
        route = client.fleetify.routes.create(
            key="key",
            driver_email="johndoe@abc.com",
            steps=[
                {
                    "arrival": 0,
                    "location": [0],
                    "type": "`start`",
                    "address": '"address": "503, Dublin Drive, Los Angeles, California - 500674",',
                    "completion_mode": "`manual`",
                    "document_template_id": "document_template_id",
                    "duration": 0,
                    "geofence_config": {
                        "radius": 0,
                        "type": "circle",
                    },
                    "meta": {
                        "customer_name": '"customer_name": "Chandler Bing"',
                        "customer_phone_number": '"customer_phone_number": "+1 707 234 1234"',
                        "instructions": '"instructions": "Customer asked not to ring the doorbell."',
                    },
                }
            ],
            distance=0,
            document_template_id='"document_template_id": "bfbc4799-bc2f-4515-9054-d888560909bf"',
            ro_request_id="ro_request_id",
            routing={
                "approaches": "`unrestricted`",
                "avoid": "`toll`",
                "hazmat_type": "`general`",
                "mode": "`car`",
                "truck_axle_load": 0,
                "truck_size": '"truck_size": "200, 210, 600"',
                "truck_weight": 0,
            },
        )
        assert_matches_type(RouteCreateResponse, route, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_create(self, client: NextbillionSDK) -> None:
        response = client.fleetify.routes.with_raw_response.create(
            key="key",
            driver_email="johndoe@abc.com",
            steps=[
                {
                    "arrival": 0,
                    "location": [0],
                    "type": "`start`",
                }
            ],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        route = response.parse()
        assert_matches_type(RouteCreateResponse, route, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_create(self, client: NextbillionSDK) -> None:
        with client.fleetify.routes.with_streaming_response.create(
            key="key",
            driver_email="johndoe@abc.com",
            steps=[
                {
                    "arrival": 0,
                    "location": [0],
                    "type": "`start`",
                }
            ],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            route = response.parse()
            assert_matches_type(RouteCreateResponse, route, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_redispatch(self, client: NextbillionSDK) -> None:
        route = client.fleetify.routes.redispatch(
            route_id="routeID",
            key="key",
            operations=[
                {
                    "data": {},
                    "operation": "`create`",
                }
            ],
        )
        assert_matches_type(RouteRedispatchResponse, route, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_redispatch_with_all_params(self, client: NextbillionSDK) -> None:
        route = client.fleetify.routes.redispatch(
            route_id="routeID",
            key="key",
            operations=[
                {
                    "data": {
                        "completion_mode": "`manual`",
                        "document_template_id": "document_template_id",
                        "step": {
                            "arrival": 0,
                            "location": [0],
                            "type": "`start`",
                            "address": '"address": "503, Dublin Drive, Los Angeles, California - 500674",',
                            "completion_mode": "`manual`",
                            "document_template_id": "document_template_id",
                            "duration": 0,
                            "geofence_config": {
                                "radius": 0,
                                "type": "circle",
                            },
                            "meta": {
                                "customer_name": '"customer_name": "Chandler Bing"',
                                "customer_phone_number": '"customer_phone_number": "+1 707 234 1234"',
                                "instructions": '"instructions": "Customer asked not to ring the doorbell."',
                            },
                        },
                        "step_id": "step_id",
                    },
                    "operation": "`create`",
                }
            ],
            distance=0,
        )
        assert_matches_type(RouteRedispatchResponse, route, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_redispatch(self, client: NextbillionSDK) -> None:
        response = client.fleetify.routes.with_raw_response.redispatch(
            route_id="routeID",
            key="key",
            operations=[
                {
                    "data": {},
                    "operation": "`create`",
                }
            ],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        route = response.parse()
        assert_matches_type(RouteRedispatchResponse, route, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_redispatch(self, client: NextbillionSDK) -> None:
        with client.fleetify.routes.with_streaming_response.redispatch(
            route_id="routeID",
            key="key",
            operations=[
                {
                    "data": {},
                    "operation": "`create`",
                }
            ],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            route = response.parse()
            assert_matches_type(RouteRedispatchResponse, route, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_redispatch(self, client: NextbillionSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `route_id` but received ''"):
            client.fleetify.routes.with_raw_response.redispatch(
                route_id="",
                key="key",
                operations=[
                    {
                        "data": {},
                        "operation": "`create`",
                    }
                ],
            )


class TestAsyncRoutes:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip()
    @parametrize
    async def test_method_create(self, async_client: AsyncNextbillionSDK) -> None:
        route = await async_client.fleetify.routes.create(
            key="key",
            driver_email="johndoe@abc.com",
            steps=[
                {
                    "arrival": 0,
                    "location": [0],
                    "type": "`start`",
                }
            ],
        )
        assert_matches_type(RouteCreateResponse, route, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncNextbillionSDK) -> None:
        route = await async_client.fleetify.routes.create(
            key="key",
            driver_email="johndoe@abc.com",
            steps=[
                {
                    "arrival": 0,
                    "location": [0],
                    "type": "`start`",
                    "address": '"address": "503, Dublin Drive, Los Angeles, California - 500674",',
                    "completion_mode": "`manual`",
                    "document_template_id": "document_template_id",
                    "duration": 0,
                    "geofence_config": {
                        "radius": 0,
                        "type": "circle",
                    },
                    "meta": {
                        "customer_name": '"customer_name": "Chandler Bing"',
                        "customer_phone_number": '"customer_phone_number": "+1 707 234 1234"',
                        "instructions": '"instructions": "Customer asked not to ring the doorbell."',
                    },
                }
            ],
            distance=0,
            document_template_id='"document_template_id": "bfbc4799-bc2f-4515-9054-d888560909bf"',
            ro_request_id="ro_request_id",
            routing={
                "approaches": "`unrestricted`",
                "avoid": "`toll`",
                "hazmat_type": "`general`",
                "mode": "`car`",
                "truck_axle_load": 0,
                "truck_size": '"truck_size": "200, 210, 600"',
                "truck_weight": 0,
            },
        )
        assert_matches_type(RouteCreateResponse, route, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncNextbillionSDK) -> None:
        response = await async_client.fleetify.routes.with_raw_response.create(
            key="key",
            driver_email="johndoe@abc.com",
            steps=[
                {
                    "arrival": 0,
                    "location": [0],
                    "type": "`start`",
                }
            ],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        route = await response.parse()
        assert_matches_type(RouteCreateResponse, route, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncNextbillionSDK) -> None:
        async with async_client.fleetify.routes.with_streaming_response.create(
            key="key",
            driver_email="johndoe@abc.com",
            steps=[
                {
                    "arrival": 0,
                    "location": [0],
                    "type": "`start`",
                }
            ],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            route = await response.parse()
            assert_matches_type(RouteCreateResponse, route, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_redispatch(self, async_client: AsyncNextbillionSDK) -> None:
        route = await async_client.fleetify.routes.redispatch(
            route_id="routeID",
            key="key",
            operations=[
                {
                    "data": {},
                    "operation": "`create`",
                }
            ],
        )
        assert_matches_type(RouteRedispatchResponse, route, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_redispatch_with_all_params(self, async_client: AsyncNextbillionSDK) -> None:
        route = await async_client.fleetify.routes.redispatch(
            route_id="routeID",
            key="key",
            operations=[
                {
                    "data": {
                        "completion_mode": "`manual`",
                        "document_template_id": "document_template_id",
                        "step": {
                            "arrival": 0,
                            "location": [0],
                            "type": "`start`",
                            "address": '"address": "503, Dublin Drive, Los Angeles, California - 500674",',
                            "completion_mode": "`manual`",
                            "document_template_id": "document_template_id",
                            "duration": 0,
                            "geofence_config": {
                                "radius": 0,
                                "type": "circle",
                            },
                            "meta": {
                                "customer_name": '"customer_name": "Chandler Bing"',
                                "customer_phone_number": '"customer_phone_number": "+1 707 234 1234"',
                                "instructions": '"instructions": "Customer asked not to ring the doorbell."',
                            },
                        },
                        "step_id": "step_id",
                    },
                    "operation": "`create`",
                }
            ],
            distance=0,
        )
        assert_matches_type(RouteRedispatchResponse, route, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_redispatch(self, async_client: AsyncNextbillionSDK) -> None:
        response = await async_client.fleetify.routes.with_raw_response.redispatch(
            route_id="routeID",
            key="key",
            operations=[
                {
                    "data": {},
                    "operation": "`create`",
                }
            ],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        route = await response.parse()
        assert_matches_type(RouteRedispatchResponse, route, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_redispatch(self, async_client: AsyncNextbillionSDK) -> None:
        async with async_client.fleetify.routes.with_streaming_response.redispatch(
            route_id="routeID",
            key="key",
            operations=[
                {
                    "data": {},
                    "operation": "`create`",
                }
            ],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            route = await response.parse()
            assert_matches_type(RouteRedispatchResponse, route, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_redispatch(self, async_client: AsyncNextbillionSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `route_id` but received ''"):
            await async_client.fleetify.routes.with_raw_response.redispatch(
                route_id="",
                key="key",
                operations=[
                    {
                        "data": {},
                        "operation": "`create`",
                    }
                ],
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["RestrictionListParams"]


class RestrictionListParams(TypedDict, total=False):
    area: Required[str]
    """Specify the area name. It represents a region where restrictions can be applied.

    _The area it belongs to. See Area API_
    """

    key: Required[str]
    """
    A key is a unique identifier that is required to authenticate a request to the
    API.
    """

    limit: Required[int]
    """The number of restrictions to be returned in the response.

    Please note that if the `limit` is set to a number more than the total number of
    available restrictions, then all restrictions would be returned together.
    """

    offset: Required[int]
    """
    An integer value indicating the number of items in the collection that need to
    be skipped in the response. Please note that the offset starts from 0, so the
    first item returned in the result would be the item at (offset + 1) position in
    collection.

    Users can use `offset` along with `limit` to implement paginated result.
    """

    mode: Literal["`0w`", "`2w`", "`3w`", "`4w`", "`6w`"]
    """Specify the modes of travel that the restriction pertains to."""

    name: str
    """The name of the restriction.

    This should be same as that provided while creating or updating the restriction.
    """

    restriction_type: Literal["`turn`", "`parking`", "`fixedspeed`", "`maxspeed`", "`closure`", "`truck`"]
    """Specify the type of restrictions to fetch."""

    source: Literal["rrt", "pbf"]
    """
    It represents where it comes from, currently the possible values include "rrt",
    "xsm"
    """

    state: Literal["`enabled`", "`disabled`", "`deleted`"]
    """This parameter is used to filter restrictions based on their state i.e.

    whether the restriction is currently enabled, disabled, or deleted. For example,
    users can retrieve a list of all the deleted restrictions by setting
    `state=deleted`.
    """

    status: Literal["`active`", "`inactive`"]
    """Restrictions can be active or inactive at a given time by virtue of their
    nature.

    For example, maximum speed limits can be active on the roads leading to schools
    during school hours and be inactive afterwards or certain road closure
    restrictions be active during holidays/concerts and be inactive otherwise.

    Use this parameter to filter the restrictions based on their status at the time
    of making the request i.e. whether they are in force or not.
    """

    transform: bool
    """a internal parameter"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from ...._utils import PropertyInfo
from .document_submission_param import DocumentSubmissionParam

__all__ = ["StepCompleteParams"]


class StepCompleteParams(TypedDict, total=False):
    route_id: Required[Annotated[str, PropertyInfo(alias="routeID")]]

    key: Required[str]
    """
    A key is a unique identifier that is required to authenticate a request to the
    API.
    """

    document: DocumentSubmissionParam
    """
    A key-value map storing form submission data, where keys correspond to field
    labels and values can be of any type depend on the type of according document
    item.
    """

    mode: str
    """Sets the status of the route step. Currently only `completed` is supported.

    Note: once marked `completed`, a step cannot transition to other statuses. You
    can only update the document afterwards.
    """

    status: str
    """Sets the status of the route step. Currently only `completed` is supported.

    Note: once marked `completed`, a step cannot transition to other statuses. You
    can only update the document afterwards.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["NamespacedApikeyCreateParams"]


class NamespacedApikeyCreateParams(TypedDict, total=False):
    key: Required[str]
    """
    A key is a unique identifier that is required to authenticate a request to the
    API.
    """

    namespace: Required[str]
    """Specify a name for the `namespace`.

    If the namespace specified is unique then a new namespace along with a new key
    is created. Whereas if the specified `namespace` is not unique, a new key will
    be created in the existing `namespace`. Please note that a `namespace` cannot be
    created using another namespace key.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["GeocodeStructuredRetrieveParams"]


class GeocodeStructuredRetrieveParams(TypedDict, total=False):
    country_code: Required[Annotated[str, PropertyInfo(alias="countryCode")]]
    """
    Specify a valid
    [ISO 3166-1 alpha-3](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-3) country
    code in which the place being searched should be located. Please note that this
    is a case-sensitive field and the country code should be in all uppercase.
    """

    key: Required[str]
    """
    A key is a unique identifier that is required to authenticate a request to the
    API.
    """

    at: str
    """Specify the center of the search context expressed as coordinates.

    Please note that one of "at", "in=circle" or "in=bbox" should be provided for
    relevant results.
    """

    city: str
    """Specify the city in which the place being searched should be located."""

    county: str
    """
    Specify the county division of the state in which the place being searched
    should be located.
    """

    house_number: Annotated[str, PropertyInfo(alias="houseNumber")]
    """Specify the house number of the place being searched."""

    in_: Annotated[str, PropertyInfo(alias="in")]
    """Search within a geographic area.

    This is a hard filter. Results will be returned if they are located within the
    specified area.

    A geographic area can be

    - a circular area, provided as latitude, longitude, and radius (an integer with
      meters as unit)

      Format: `circle:{latitude},{longitude};r={radius}`

    - a bounding box, provided as _west longitude_, _south latitude_, _east
      longitude_, _north latitude_

      Format:
      `bbox:{west longitude},{south latitude},{east longitude},{north latitude}`

    Please provide one of 'at', 'in=circle' or 'in=bbox' input for a relevant
    result.
    """

    limit: int
    """Sets the maximum number of results to be returned."""

    postal_code: Annotated[str, PropertyInfo(alias="postalCode")]
    """Specify the postal code in which the place being searched should be located."""

    state: str
    """
    Specify the state division of the country in which the place being searched
    should be located.
    """

    street: str
    """
    Specify the name of the street in which the place being searched should be
    located.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from tests.utils import assert_matches_type
from nextbillion_sdk import NextbillionSDK, AsyncNextbillionSDK
from nextbillion_sdk.types import (
    GeofenceListResponse,
    GeofenceCreateResponse,
    GeofenceContainsResponse,
    GeofenceRetrieveResponse,
)
from nextbillion_sdk.types.skynet import SimpleResp

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestGeofence:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    def test_method_create(self, client: NextbillionSDK) -> None:
        geofence = client.geofence.create(
            key="key=API_KEY",
            type="`circle`",
        )
        assert_matches_type(GeofenceCreateResponse, geofence, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_create_with_all_params(self, client: NextbillionSDK) -> None:
        geofence = client.geofence.create(
            key="key=API_KEY",
            type="`circle`",
            circle={
                "center": {
                    "lat": 0,
                    "lon": 0,
                },
                "radius": 0,
            },
            custom_id="custom_id",
            isochrone={
                "coordinates": '"coordinates": "13.25805884,77.91083661"',
                "contours_meter": 0,
                "contours_minute": 0,
                "denoise": 0,
                "departure_time": 0,
                "mode": "`car`",
            },
            meta_data='{\n  "country": "USA",\n  "state": "California"\n}',
            name='"name":"Los Angeles Downtown"',
            polygon={
                "geojson": {
                    "coordinates": [[0]],
                    "type": "type",
                }
            },
            tags=['"tags":["tags_1", "O69Am2Y4KL8q5Y5JuD-Fy-tdtEpkTRQo_ZYIK7"]'],
        )
        assert_matches_type(GeofenceCreateResponse, geofence, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_create(self, client: NextbillionSDK) -> None:
        response = client.geofence.with_raw_response.create(
            key="key=API_KEY",
            type="`circle`",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        geofence = response.parse()
        assert_matches_type(GeofenceCreateResponse, geofence, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_create(self, client: NextbillionSDK) -> None:
        with client.geofence.with_streaming_response.create(
            key="key=API_KEY",
            type="`circle`",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            geofence = response.parse()
            assert_matches_type(GeofenceCreateResponse, geofence, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_retrieve(self, client: NextbillionSDK) -> None:
        geofence = client.geofence.retrieve(
            id="id",
            key="key=API_KEY",
        )
        assert_matches_type(GeofenceRetrieveResponse, geofence, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_retrieve(self, client: NextbillionSDK) -> None:
        response = client.geofence.with_raw_response.retrieve(
            id="id",
            key="key=API_KEY",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        geofence = response.parse()
        assert_matches_type(GeofenceRetrieveResponse, geofence, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_retrieve(self, client: NextbillionSDK) -> None:
        with client.geofence.with_streaming_response.retrieve(
            id="id",
            key="key=API_KEY",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            geofence = response.parse()
            assert_matches_type(GeofenceRetrieveResponse, geofence, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_retrieve(self, client: NextbillionSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.geofence.with_raw_response.retrieve(
                id="",
                key="key=API_KEY",
            )

    @pytest.mark.skip()
    @parametrize
    def test_method_update(self, client: NextbillionSDK) -> None:
        geofence = client.geofence.update(
            id="id",
            key="key=API_KEY",
        )
        assert_matches_type(SimpleResp, geofence, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_update_with_all_params(self, client: NextbillionSDK) -> None:
        geofence = client.geofence.update(
            id="id",
            key="key=API_KEY",
            circle={
                "center": {
                    "lat": 0,
                    "lon": 0,
                },
                "radius": 0,
            },
            isochrone={
                "contours_meter": 0,
                "contours_minute": 0,
                "coordinates": '"coordinates": "13.25805884388484,77.91083661048299"',
                "denoise": 0,
                "departure_time": 0,
                "mode": "“mode”:”car”",
            },
            meta_data="",
            name='"name":"Los Angeles Downtown"',
            polygon={
                "geojson": {
                    "geometry": [[0]],
                    "type": "type",
                }
            },
            tags=['"tags":["tags_1", "O69Am2Y4KL8q5Y5JuD-Fy-tdtEpkTRQo_ZYIK7"]'],
            type="`circle`",
        )
        assert_matches_type(SimpleResp, geofence, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_update(self, client: NextbillionSDK) -> None:
        response = client.geofence.with_raw_response.update(
            id="id",
            key="key=API_KEY",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        geofence = response.parse()
        assert_matches_type(SimpleResp, geofence, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_update(self, client: NextbillionSDK) -> None:
        with client.geofence.with_streaming_response.update(
            id="id",
            key="key=API_KEY",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            geofence = response.parse()
            assert_matches_type(SimpleResp, geofence, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_update(self, client: NextbillionSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.geofence.with_raw_response.update(
                id="",
                key="key=API_KEY",
            )

    @pytest.mark.skip()
    @parametrize
    def test_method_list(self, client: NextbillionSDK) -> None:
        geofence = client.geofence.list(
            key="key=API_KEY",
        )
        assert_matches_type(GeofenceListResponse, geofence, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_list_with_all_params(self, client: NextbillionSDK) -> None:
        geofence = client.geofence.list(
            key="key=API_KEY",
            pn=0,
            ps=100,
            tags="tags=tags_1,O69Am2Y4KL8q5Y5JuD-Fy-tdtEpkTRQo_ZYIK7",
        )
        assert_matches_type(GeofenceListResponse, geofence, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_list(self, client: NextbillionSDK) -> None:
        response = client.geofence.with_raw_response.list(
            key="key=API_KEY",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        geofence = response.parse()
        assert_matches_type(GeofenceListResponse, geofence, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_list(self, client: NextbillionSDK) -> None:
        with client.geofence.with_streaming_response.list(
            key="key=API_KEY",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            geofence = response.parse()
            assert_matches_type(GeofenceListResponse, geofence, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_delete(self, client: NextbillionSDK) -> None:
        geofence = client.geofence.delete(
            id="id",
            key="key=API_KEY",
        )
        assert_matches_type(SimpleResp, geofence, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_delete(self, client: NextbillionSDK) -> None:
        response = client.geofence.with_raw_response.delete(
            id="id",
            key="key=API_KEY",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        geofence = response.parse()
        assert_matches_type(SimpleResp, geofence, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_delete(self, client: NextbillionSDK) -> None:
        with client.geofence.with_streaming_response.delete(
            id="id",
            key="key=API_KEY",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            geofence = response.parse()
            assert_matches_type(SimpleResp, geofence, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_delete(self, client: NextbillionSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.geofence.with_raw_response.delete(
                id="",
                key="key=API_KEY",
            )

    @pytest.mark.skip()
    @parametrize
    def test_method_contains(self, client: NextbillionSDK) -> None:
        geofence = client.geofence.contains(
            key="key=API_KEY",
            locations="13.25805884388484,77.91083661048299|13.25805884388484,77.91083661048299",
        )
        assert_matches_type(GeofenceContainsResponse, geofence, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_contains_with_all_params(self, client: NextbillionSDK) -> None:
        geofence = client.geofence.contains(
            key="key=API_KEY",
            locations="13.25805884388484,77.91083661048299|13.25805884388484,77.91083661048299",
            geofences="geofences=80d1fa55-6287-4da0-93ac-2fc162d0a228,70d1fa55-1287-4da0-93ac-2fc162d0a228",
            verbose="verbose=true",
        )
        assert_matches_type(GeofenceContainsResponse, geofence, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_contains(self, client: NextbillionSDK) -> None:
        response = client.geofence.with_raw_response.contains(
            key="key=API_KEY",
            locations="13.25805884388484,77.91083661048299|13.25805884388484,77.91083661048299",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        geofence = response.parse()
        assert_matches_type(GeofenceContainsResponse, geofence, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_contains(self, client: NextbillionSDK) -> None:
        with client.geofence.with_streaming_response.contains(
            key="key=API_KEY",
            locations="13.25805884388484,77.91083661048299|13.25805884388484,77.91083661048299",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            geofence = response.parse()
            assert_matches_type(GeofenceContainsResponse, geofence, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncGeofence:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip()
    @parametrize
    async def test_method_create(self, async_client: AsyncNextbillionSDK) -> None:
        geofence = await async_client.geofence.create(
            key="key=API_KEY",
            type="`circle`",
        )
        assert_matches_type(GeofenceCreateResponse, geofence, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncNextbillionSDK) -> None:
        geofence = await async_client.geofence.create(
            key="key=API_KEY",
            type="`circle`",
            circle={
                "center": {
                    "lat": 0,
                    "lon": 0,
                },
                "radius": 0,
            },
            custom_id="custom_id",
            isochrone={
                "coordinates": '"coordinates": "13.25805884,77.91083661"',
                "contours_meter": 0,
                "contours_minute": 0,
                "denoise": 0,
                "departure_time": 0,
                "mode": "`car`",
            },
            meta_data='{\n  "country": "USA",\n  "state": "California"\n}',
            name='"name":"Los Angeles Downtown"',
            polygon={
                "geojson": {
                    "coordinates": [[0]],
                    "type": "type",
                }
            },
            tags=['"tags":["tags_1", "O69Am2Y4KL8q5Y5JuD-Fy-tdtEpkTRQo_ZYIK7"]'],
        )
        assert_matches_type(GeofenceCreateResponse, geofence, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncNextbillionSDK) -> None:
        response = await async_client.geofence.with_raw_response.create(
            key="key=API_KEY",
            type="`circle`",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        geofence = await response.parse()
        assert_matches_type(GeofenceCreateResponse, geofence, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncNextbillionSDK) -> None:
        async with async_client.geofence.with_streaming_response.create(
            key="key=API_KEY",
            type="`circle`",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            geofence = await response.parse()
            assert_matches_type(GeofenceCreateResponse, geofence, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncNextbillionSDK) -> None:
        geofence = await async_client.geofence.retrieve(
            id="id",
            key="key=API_KEY",
        )
        assert_matches_type(GeofenceRetrieveResponse, geofence, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncNextbillionSDK) -> None:
        response = await async_client.geofence.with_raw_response.retrieve(
            id="id",
            key="key=API_KEY",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        geofence = await response.parse()
        assert_matches_type(GeofenceRetrieveResponse, geofence, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncNextbillionSDK) -> None:
        async with async_client.geofence.with_streaming_response.retrieve(
            id="id",
            key="key=API_KEY",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            geofence = await response.parse()
            assert_matches_type(GeofenceRetrieveResponse, geofence, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncNextbillionSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.geofence.with_raw_response.retrieve(
                id="",
                key="key=API_KEY",
            )

    @pytest.mark.skip()
    @parametrize
    async def test_method_update(self, async_client: AsyncNextbillionSDK) -> None:
        geofence = await async_client.geofence.update(
            id="id",
            key="key=API_KEY",
        )
        assert_matches_type(SimpleResp, geofence, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncNextbillionSDK) -> None:
        geofence = await async_client.geofence.update(
            id="id",
            key="key=API_KEY",
            circle={
                "center": {
                    "lat": 0,
                    "lon": 0,
                },
                "radius": 0,
            },
            isochrone={
                "contours_meter": 0,
                "contours_minute": 0,
                "coordinates": '"coordinates": "13.25805884388484,77.91083661048299"',
                "denoise": 0,
                "departure_time": 0,
                "mode": "“mode”:”car”",
            },
            meta_data="",
            name='"name":"Los Angeles Downtown"',
            polygon={
                "geojson": {
                    "geometry": [[0]],
                    "type": "type",
                }
            },
            tags=['"tags":["tags_1", "O69Am2Y4KL8q5Y5JuD-Fy-tdtEpkTRQo_ZYIK7"]'],
            type="`circle`",
        )
        assert_matches_type(SimpleResp, geofence, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncNextbillionSDK) -> None:
        response = await async_client.geofence.with_raw_response.update(
            id="id",
            key="key=API_KEY",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        geofence = await response.parse()
        assert_matches_type(SimpleResp, geofence, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncNextbillionSDK) -> None:
        async with async_client.geofence.with_streaming_response.update(
            id="id",
            key="key=API_KEY",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            geofence = await response.parse()
            assert_matches_type(SimpleResp, geofence, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_update(self, async_client: AsyncNextbillionSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.geofence.with_raw_response.update(
                id="",
                key="key=API_KEY",
            )

    @pytest.mark.skip()
    @parametrize
    async def test_method_list(self, async_client: AsyncNextbillionSDK) -> None:
        geofence = await async_client.geofence.list(
            key="key=API_KEY",
        )
        assert_matches_type(GeofenceListResponse, geofence, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncNextbillionSDK) -> None:
        geofence = await async_client.geofence.list(
            key="key=API_KEY",
            pn=0,
            ps=100,
            tags="tags=tags_1,O69Am2Y4KL8q5Y5JuD-Fy-tdtEpkTRQo_ZYIK7",
        )
        assert_matches_type(GeofenceListResponse, geofence, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncNextbillionSDK) -> None:
        response = await async_client.geofence.with_raw_response.list(
            key="key=API_KEY",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        geofence = await response.parse()
        assert_matches_type(GeofenceListResponse, geofence, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncNextbillionSDK) -> None:
        async with async_client.geofence.with_streaming_response.list(
            key="key=API_KEY",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            geofence = await response.parse()
            assert_matches_type(GeofenceListResponse, geofence, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_delete(self, async_client: AsyncNextbillionSDK) -> None:
        geofence = await async_client.geofence.delete(
            id="id",
            key="key=API_KEY",
        )
        assert_matches_type(SimpleResp, geofence, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncNextbillionSDK) -> None:
        response = await async_client.geofence.with_raw_response.delete(
            id="id",
            key="key=API_KEY",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        geofence = await response.parse()
        assert_matches_type(SimpleResp, geofence, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncNextbillionSDK) -> None:
        async with async_client.geofence.with_streaming_response.delete(
            id="id",
            key="key=API_KEY",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            geofence = await response.parse()
            assert_matches_type(SimpleResp, geofence, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncNextbillionSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.geofence.with_raw_response.delete(
                id="",
                key="key=API_KEY",
            )

    @pytest.mark.skip()
    @parametrize
    async def test_method_contains(self, async_client: AsyncNextbillionSDK) -> None:
        geofence = await async_client.geofence.contains(
            key="key=API_KEY",
            locations="13.25805884388484,77.91083661048299|13.25805884388484,77.91083661048299",
        )
        assert_matches_type(GeofenceContainsResponse, geofence, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_contains_with_all_params(self, async_client: AsyncNextbillionSDK) -> None:
        geofence = await async_client.geofence.contains(
            key="key=API_KEY",
            locations="13.25805884388484,77.91083661048299|13.25805884388484,77.91083661048299",
            geofences="geofences=80d1fa55-6287-4da0-93ac-2fc162d0a228,70d1fa55-1287-4da0-93ac-2fc162d0a228",
            verbose="verbose=true",
        )
        assert_matches_type(GeofenceContainsResponse, geofence, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_contains(self, async_client: AsyncNextbillionSDK) -> None:
        response = await async_client.geofence.with_raw_response.contains(
            key="key=API_KEY",
            locations="13.25805884388484,77.91083661048299|13.25805884388484,77.91083661048299",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        geofence = await response.parse()
        assert_matches_type(GeofenceContainsResponse, geofence, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_contains(self, async_client: AsyncNextbillionSDK) -> None:
        async with async_client.geofence.with_streaming_response.contains(
            key="key=API_KEY",
            locations="13.25805884388484,77.91083661048299|13.25805884388484,77.91083661048299",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            geofence = await response.parse()
            assert_matches_type(GeofenceContainsResponse, geofence, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = [
    "OptimizationComputeResponse",
    "Location",
    "Trip",
    "TripGeojson",
    "TripLeg",
    "TripLegStep",
    "TripLegStepGeojson",
    "Waypoint",
    "WaypointLocation",
]


class Location(BaseModel):
    latitude: Optional[float] = None
    """Latitude coordinate of the location."""

    longitude: Optional[float] = None
    """Longitude coordinate of the location."""


class TripGeojson(BaseModel):
    geometry: Optional[str] = None
    """The encoded geometry of the geojson in the `trip`."""

    properties: Optional[str] = None
    """Additional properties associated with the `trip`."""

    type: Optional[
        Literal[
            "Point",
            "MultiPoint",
            "LineString",
            "MultiLineString",
            "Polygon",
            "MultiPolygon",
            "GeometryCollection",
            "Feature",
            "FeatureCollection",
            "Link",
        ]
    ] = None
    """The type of the GeoJSON object."""


class TripLegStepGeojson(BaseModel):
    geometry: Optional[str] = None
    """The encoded geometry of the geojson in the `step`."""

    properties: Optional[str] = None
    """Additional properties associated with the `step`."""

    type: Optional[
        Literal[
            "Point",
            "MultiPoint",
            "LineString",
            "MultiLineString",
            "Polygon",
            "MultiPolygon",
            "GeometryCollection",
            "Feature",
            "FeatureCollection",
            "Link",
        ]
    ] = None
    """The type of the GeoJSON object."""


class TripLegStep(BaseModel):
    distance: Optional[float] = None
    """Distance of the `step` object in meters."""

    duration: Optional[float] = None
    """Duration of the `step` object in seconds."""

    geojson: Optional[TripLegStepGeojson] = None
    """The GeoJSON representation of the `step`."""

    geometry: Optional[str] = None
    """Encoded geometry of the `step` in the selected format."""


class TripLeg(BaseModel):
    distance: Optional[float] = None
    """Distance of leg in metres."""

    duration: Optional[float] = None
    """Duration of leg in seconds."""

    steps: Optional[List[TripLegStep]] = None
    """An array of step objects."""

    summary: Optional[str] = None
    """Summary of the `leg` object."""


class Trip(BaseModel):
    distance: Optional[float] = None
    """Distance of the trip in meters."""

    duration: Optional[float] = None
    """Duration of the trip in seconds"""

    geojson: Optional[TripGeojson] = None
    """The GeoJSON representation of the route."""

    geometry: Optional[str] = None
    """`polyline` or `polyline6` format of route geometry."""

    legs: Optional[List[TripLeg]] = None


class WaypointLocation(BaseModel):
    latitude: Optional[float] = None
    """Latitude coordinate of the waypoint."""

    longitude: Optional[float] = None
    """Longitude coordinate of the waypoint."""


class Waypoint(BaseModel):
    location: Optional[WaypointLocation] = None
    """Describes the location of the waypoint."""

    name: Optional[str] = None
    """Name of the waypoint."""

    trips_index: Optional[int] = None
    """Denotes the ID of a trip. Starts with 0."""

    waypoint_index: Optional[int] = None
    """Denotes the id of a waypoint.

    The first waypoint is denoted with 0. And onwards with 1, 2 etc.
    """


class OptimizationComputeResponse(BaseModel):
    code: Optional[str] = None
    """A string indicating the state of the response.

    This is a separate code than the HTTP status code. On normal valid responses,
    the value will be `Ok`.
    """

    location: Optional[Location] = None
    """Contains the latitude and longitude of a location"""

    trips: Optional[List[Trip]] = None
    """An array of 0 or 1 trip objects. Each object has the following schema."""

    waypoints: Optional[List[Waypoint]] = None
    """Each waypoint is an input coordinate snapped to the road and path network."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable

import httpx

from ..types import geocode_retrieve_params, geocode_batch_create_params, geocode_structured_retrieve_params
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.geocode_retrieve_response import GeocodeRetrieveResponse
from ..types.geocode_batch_create_response import GeocodeBatchCreateResponse
from ..types.geocode_structured_retrieve_response import GeocodeStructuredRetrieveResponse

__all__ = ["GeocodeResource", "AsyncGeocodeResource"]


class GeocodeResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> GeocodeResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#accessing-raw-response-data-eg-headers
        """
        return GeocodeResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> GeocodeResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#with_streaming_response
        """
        return GeocodeResourceWithStreamingResponse(self)

    def retrieve(
        self,
        *,
        key: str,
        q: str,
        at: str | NotGiven = NOT_GIVEN,
        in_: str | NotGiven = NOT_GIVEN,
        lang: str | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> GeocodeRetrieveResponse:
        """
        Geocode

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          q: Specify the free-text search query.

              Please note that whitespace, urls, email addresses, or other out-of-scope
              queries will yield no results.

          at: Specify the center of the search context expressed as coordinates.

              Please note that one of "at", "in=circle" or "in=bbox" should be provided for
              relevant results.

          in_: Search within a geographic area. This is a hard filter. Results will be returned
              if they are located within the specified area.

              A geographic area can be

              - a country (or multiple countries), provided as comma-separated
                [ISO 3166-1 alpha-3](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-3) country
                codes

                The country codes are to be provided in all uppercase.

                Format: `countryCode:{countryCode}[,{countryCode}]`

              - a circular area, provided as latitude, longitude, and radius (an integer with
                meters as unit)

                Format: `circle:{latitude},{longitude};r={radius}`

              - a bounding box, provided as _west longitude_, _south latitude_, _east
                longitude_, _north latitude_

                Format:
                `bbox:{west longitude},{south latitude},{east longitude},{north latitude}`

              Please provide one of 'at', 'in=circle' or 'in=bbox' input for a relevant
              result.

          lang: Select the language to be used for result rendering from a list of
              [BCP 47](https://en.wikipedia.org/wiki/IETF_language_tag) compliant language
              codes.

          limit: Sets the maximum number of results to be returned.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/geocode",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "key": key,
                        "q": q,
                        "at": at,
                        "in_": in_,
                        "lang": lang,
                        "limit": limit,
                    },
                    geocode_retrieve_params.GeocodeRetrieveParams,
                ),
            ),
            cast_to=GeocodeRetrieveResponse,
        )

    def batch_create(
        self,
        *,
        key: str,
        body: Iterable[geocode_batch_create_params.Body],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> GeocodeBatchCreateResponse:
        """
        Batch Geocode

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/geocode/batch",
            body=maybe_transform(body, Iterable[geocode_batch_create_params.Body]),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"key": key}, geocode_batch_create_params.GeocodeBatchCreateParams),
            ),
            cast_to=GeocodeBatchCreateResponse,
        )

    def structured_retrieve(
        self,
        *,
        country_code: str,
        key: str,
        at: str | NotGiven = NOT_GIVEN,
        city: str | NotGiven = NOT_GIVEN,
        county: str | NotGiven = NOT_GIVEN,
        house_number: str | NotGiven = NOT_GIVEN,
        in_: str | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        postal_code: str | NotGiven = NOT_GIVEN,
        state: str | NotGiven = NOT_GIVEN,
        street: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> GeocodeStructuredRetrieveResponse:
        """
        Structured Geocode

        Args:
          country_code: Specify a valid
              [ISO 3166-1 alpha-3](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-3) country
              code in which the place being searched should be located. Please note that this
              is a case-sensitive field and the country code should be in all uppercase.

          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          at: Specify the center of the search context expressed as coordinates.

              Please note that one of "at", "in=circle" or "in=bbox" should be provided for
              relevant results.

          city: Specify the city in which the place being searched should be located.

          county: Specify the county division of the state in which the place being searched
              should be located.

          house_number: Specify the house number of the place being searched.

          in_: Search within a geographic area. This is a hard filter. Results will be returned
              if they are located within the specified area.

              A geographic area can be

              - a circular area, provided as latitude, longitude, and radius (an integer with
                meters as unit)

                Format: `circle:{latitude},{longitude};r={radius}`

              - a bounding box, provided as _west longitude_, _south latitude_, _east
                longitude_, _north latitude_

                Format:
                `bbox:{west longitude},{south latitude},{east longitude},{north latitude}`

              Please provide one of 'at', 'in=circle' or 'in=bbox' input for a relevant
              result.

          limit: Sets the maximum number of results to be returned.

          postal_code: Specify the postal code in which the place being searched should be located.

          state: Specify the state division of the country in which the place being searched
              should be located.

          street: Specify the name of the street in which the place being searched should be
              located.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/geocode/structured",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "country_code": country_code,
                        "key": key,
                        "at": at,
                        "city": city,
                        "county": county,
                        "house_number": house_number,
                        "in_": in_,
                        "limit": limit,
                        "postal_code": postal_code,
                        "state": state,
                        "street": street,
                    },
                    geocode_structured_retrieve_params.GeocodeStructuredRetrieveParams,
                ),
            ),
            cast_to=GeocodeStructuredRetrieveResponse,
        )


class AsyncGeocodeResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncGeocodeResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncGeocodeResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncGeocodeResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#with_streaming_response
        """
        return AsyncGeocodeResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        *,
        key: str,
        q: str,
        at: str | NotGiven = NOT_GIVEN,
        in_: str | NotGiven = NOT_GIVEN,
        lang: str | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> GeocodeRetrieveResponse:
        """
        Geocode

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          q: Specify the free-text search query.

              Please note that whitespace, urls, email addresses, or other out-of-scope
              queries will yield no results.

          at: Specify the center of the search context expressed as coordinates.

              Please note that one of "at", "in=circle" or "in=bbox" should be provided for
              relevant results.

          in_: Search within a geographic area. This is a hard filter. Results will be returned
              if they are located within the specified area.

              A geographic area can be

              - a country (or multiple countries), provided as comma-separated
                [ISO 3166-1 alpha-3](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-3) country
                codes

                The country codes are to be provided in all uppercase.

                Format: `countryCode:{countryCode}[,{countryCode}]`

              - a circular area, provided as latitude, longitude, and radius (an integer with
                meters as unit)

                Format: `circle:{latitude},{longitude};r={radius}`

              - a bounding box, provided as _west longitude_, _south latitude_, _east
                longitude_, _north latitude_

                Format:
                `bbox:{west longitude},{south latitude},{east longitude},{north latitude}`

              Please provide one of 'at', 'in=circle' or 'in=bbox' input for a relevant
              result.

          lang: Select the language to be used for result rendering from a list of
              [BCP 47](https://en.wikipedia.org/wiki/IETF_language_tag) compliant language
              codes.

          limit: Sets the maximum number of results to be returned.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/geocode",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "key": key,
                        "q": q,
                        "at": at,
                        "in_": in_,
                        "lang": lang,
                        "limit": limit,
                    },
                    geocode_retrieve_params.GeocodeRetrieveParams,
                ),
            ),
            cast_to=GeocodeRetrieveResponse,
        )

    async def batch_create(
        self,
        *,
        key: str,
        body: Iterable[geocode_batch_create_params.Body],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> GeocodeBatchCreateResponse:
        """
        Batch Geocode

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/geocode/batch",
            body=await async_maybe_transform(body, Iterable[geocode_batch_create_params.Body]),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"key": key}, geocode_batch_create_params.GeocodeBatchCreateParams),
            ),
            cast_to=GeocodeBatchCreateResponse,
        )

    async def structured_retrieve(
        self,
        *,
        country_code: str,
        key: str,
        at: str | NotGiven = NOT_GIVEN,
        city: str | NotGiven = NOT_GIVEN,
        county: str | NotGiven = NOT_GIVEN,
        house_number: str | NotGiven = NOT_GIVEN,
        in_: str | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        postal_code: str | NotGiven = NOT_GIVEN,
        state: str | NotGiven = NOT_GIVEN,
        street: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> GeocodeStructuredRetrieveResponse:
        """
        Structured Geocode

        Args:
          country_code: Specify a valid
              [ISO 3166-1 alpha-3](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-3) country
              code in which the place being searched should be located. Please note that this
              is a case-sensitive field and the country code should be in all uppercase.

          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          at: Specify the center of the search context expressed as coordinates.

              Please note that one of "at", "in=circle" or "in=bbox" should be provided for
              relevant results.

          city: Specify the city in which the place being searched should be located.

          county: Specify the county division of the state in which the place being searched
              should be located.

          house_number: Specify the house number of the place being searched.

          in_: Search within a geographic area. This is a hard filter. Results will be returned
              if they are located within the specified area.

              A geographic area can be

              - a circular area, provided as latitude, longitude, and radius (an integer with
                meters as unit)

                Format: `circle:{latitude},{longitude};r={radius}`

              - a bounding box, provided as _west longitude_, _south latitude_, _east
                longitude_, _north latitude_

                Format:
                `bbox:{west longitude},{south latitude},{east longitude},{north latitude}`

              Please provide one of 'at', 'in=circle' or 'in=bbox' input for a relevant
              result.

          limit: Sets the maximum number of results to be returned.

          postal_code: Specify the postal code in which the place being searched should be located.

          state: Specify the state division of the country in which the place being searched
              should be located.

          street: Specify the name of the street in which the place being searched should be
              located.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/geocode/structured",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "country_code": country_code,
                        "key": key,
                        "at": at,
                        "city": city,
                        "county": county,
                        "house_number": house_number,
                        "in_": in_,
                        "limit": limit,
                        "postal_code": postal_code,
                        "state": state,
                        "street": street,
                    },
                    geocode_structured_retrieve_params.GeocodeStructuredRetrieveParams,
                ),
            ),
            cast_to=GeocodeStructuredRetrieveResponse,
        )


class GeocodeResourceWithRawResponse:
    def __init__(self, geocode: GeocodeResource) -> None:
        self._geocode = geocode

        self.retrieve = to_raw_response_wrapper(
            geocode.retrieve,
        )
        self.batch_create = to_raw_response_wrapper(
            geocode.batch_create,
        )
        self.structured_retrieve = to_raw_response_wrapper(
            geocode.structured_retrieve,
        )


class AsyncGeocodeResourceWithRawResponse:
    def __init__(self, geocode: AsyncGeocodeResource) -> None:
        self._geocode = geocode

        self.retrieve = async_to_raw_response_wrapper(
            geocode.retrieve,
        )
        self.batch_create = async_to_raw_response_wrapper(
            geocode.batch_create,
        )
        self.structured_retrieve = async_to_raw_response_wrapper(
            geocode.structured_retrieve,
        )


class GeocodeResourceWithStreamingResponse:
    def __init__(self, geocode: GeocodeResource) -> None:
        self._geocode = geocode

        self.retrieve = to_streamed_response_wrapper(
            geocode.retrieve,
        )
        self.batch_create = to_streamed_response_wrapper(
            geocode.batch_create,
        )
        self.structured_retrieve = to_streamed_response_wrapper(
            geocode.structured_retrieve,
        )


class AsyncGeocodeResourceWithStreamingResponse:
    def __init__(self, geocode: AsyncGeocodeResource) -> None:
        self._geocode = geocode

        self.retrieve = async_to_streamed_response_wrapper(
            geocode.retrieve,
        )
        self.batch_create = async_to_streamed_response_wrapper(
            geocode.batch_create,
        )
        self.structured_retrieve = async_to_streamed_response_wrapper(
            geocode.structured_retrieve,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Literal

import httpx

from ...._types import NOT_GIVEN, Body, Query, Headers, NoneType, NotGiven
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.fleetify.routes import (
    RouteStepCompletionMode,
    step_create_params,
    step_delete_params,
    step_update_params,
    step_complete_params,
)
from ....types.fleetify.routes.step_create_response import StepCreateResponse
from ....types.fleetify.routes.step_delete_response import StepDeleteResponse
from ....types.fleetify.routes.step_update_response import StepUpdateResponse
from ....types.fleetify.routes.document_submission_param import DocumentSubmissionParam
from ....types.fleetify.routes.route_step_completion_mode import RouteStepCompletionMode
from ....types.fleetify.routes.route_step_geofence_config_param import RouteStepGeofenceConfigParam

__all__ = ["StepsResource", "AsyncStepsResource"]


class StepsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> StepsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#accessing-raw-response-data-eg-headers
        """
        return StepsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> StepsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#with_streaming_response
        """
        return StepsResourceWithStreamingResponse(self)

    def create(
        self,
        route_id: str,
        *,
        key: str,
        arrival: int,
        location: Iterable[float],
        position: int,
        type: Literal["`start`", "`job`", "`pickup`", "`delivery`", "`break`", "`layover`", "`end`"],
        address: str | NotGiven = NOT_GIVEN,
        completion_mode: RouteStepCompletionMode | NotGiven = NOT_GIVEN,
        document_template_id: str | NotGiven = NOT_GIVEN,
        duration: int | NotGiven = NOT_GIVEN,
        geofence_config: RouteStepGeofenceConfigParam | NotGiven = NOT_GIVEN,
        meta: step_create_params.Meta | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> StepCreateResponse:
        """
        Insert a new step

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          arrival: Specify the scheduled arrival time of the driver, as an UNIX timestamp in
              seconds, at the step. Please note that:

              - Arrival time for each step should be equal to or greater than the previous
                step.
              - Past times can not be provided.
              - The time provided is used only for informative display on the driver app and
                it does not impact or get affected by the route generated.

          location: Specify the location coordinates where the steps should be performed in
              `[latitude, longitude]`.

          position: Indicates the index at which to insert the step, starting from 0 up to the total
              number of steps in the route.

          type: Specify the step type. It can belong to one of the following: `start`, `job` ,
              `pickup`, `delivery`, `end`. A `duration` is mandatory when the step type is
              either `layover` or a `break`.

          address: Specify the postal address for the step.

          completion_mode: Specify the mode of completion to be used for the step. Currently, following
              values are allowed:

              - `manual`: Steps must be marked as completed manually through the Driver App.
              - `geofence`: Steps are marked as completed automatically based on the entry
                conditions and geofence specified.
              - `geofence_manual_fallback`: Steps will be marked as completed automatically
                based on geofence and entry condition configurations but there will also be a
                provision for manually updating the status in case, geofence detection fails.

          document_template_id: Specify the ID of the document template to be used for collecting proof of
              completion for the step. If not specified, the document template specified at
              the route level will be used for the step. Use the
              [Documents API](https://docs.nextbillion.ai/docs/dispatches/documents-api) to
              create, read and manage the document templates.

              Please note that the document template ID can not be assigned to following step
              types - `start`, `end`, `break`, `layover`.

          duration: Specify the duration of the `layover` or `break` type steps, in seconds. Please
              note it is mandatory when step type is either "layover" or "break".

          geofence_config: Specify the configurations of the geofence which will be used to detect presence
              of the driver and complete the tasks automatically. Please note that this
              attribute is required when `completion_mode` is either "geofence" or
              "geofence_manual_fallback".

          meta: An object to specify any additional details about the task to be associated with
              the step in the response. The information provided here will be available on the
              Driver's app under step details. This attribute can be used to provide context
              about or instructions to the driver for performing the task

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not route_id:
            raise ValueError(f"Expected a non-empty value for `route_id` but received {route_id!r}")
        return self._post(
            f"/fleetify/routes/{route_id}/steps",
            body=maybe_transform(
                {
                    "arrival": arrival,
                    "location": location,
                    "position": position,
                    "type": type,
                    "address": address,
                    "completion_mode": completion_mode,
                    "document_template_id": document_template_id,
                    "duration": duration,
                    "geofence_config": geofence_config,
                    "meta": meta,
                },
                step_create_params.StepCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"key": key}, step_create_params.StepCreateParams),
            ),
            cast_to=StepCreateResponse,
        )

    def update(
        self,
        step_id: str,
        *,
        route_id: str,
        key: str,
        arrival: int,
        position: int,
        address: str | NotGiven = NOT_GIVEN,
        completion_mode: RouteStepCompletionMode | NotGiven = NOT_GIVEN,
        document_template_id: str | NotGiven = NOT_GIVEN,
        duration: int | NotGiven = NOT_GIVEN,
        geofence_config: RouteStepGeofenceConfigParam | NotGiven = NOT_GIVEN,
        location: Iterable[float] | NotGiven = NOT_GIVEN,
        meta: step_update_params.Meta | NotGiven = NOT_GIVEN,
        type: Literal["`start`", "`job`", "`pickup`", "`delivery`", "`break`", "`layover`", "`end`"]
        | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> StepUpdateResponse:
        """
        Update a step

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          arrival: Specify the scheduled arrival time of the driver, as an UNIX timestamp in
              seconds, at the step. Please note that:

              - Arrival time for each step should be equal to or greater than the previous
                step.
              - Past times can not be provided.
              - The time provided is used only for informative display on the driver app and
                it does not impact or get affected by the route generated.

          position: Specify the new position of the step. Provide a position different from the
              current position of the step to update sequence in which the step get completed.

          address: Specify the postal address for the step.

          completion_mode: Specify the mode of completion to be used for the step. Currently, following
              values are allowed:

              - `manual`: Steps must be marked as completed manually through the Driver App.
              - `geofence`: Steps are marked as completed automatically based on the entry
                conditions and geofence specified.
              - `geofence_manual_fallback`: Steps will be marked as completed automatically
                based on geofence and entry condition configurations but there will also be a
                provision for manually updating the status in case, geofence detection fails.

          document_template_id: Update the ID of the document template to be used for collecting proof of
              completion for the step. If an empty string "" is provided, the current document
              template associated to the step will be removed.

          duration: Specify the duration of the `layover` or `break` type steps, in seconds. Please
              note it is mandatory when step type is either "layover" or "break".

          geofence_config: Specify the configurations of the geofence which will be used to detect presence
              of the driver and complete the tasks automatically. Please note that this
              attribute is required when `completion_mode` is either "geofence" or
              "geofence_manual_fallback".

          location: Specify the location coordinates where the steps should be performed in
              `[latitude, longitude]`.

          meta: An object to specify any additional details about the task to be associated with
              the step in the response. The information provided here will be available on the
              Driver's app under step details. This attribute can be used to provide context
              about or instructions to the driver for performing the task

          type: Specify the step type. It can belong to one of the following: `start`, `job` ,
              `pickup`, `delivery`, `end`. A `duration` is mandatory when the step type is
              either `layover` or a `break`.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not route_id:
            raise ValueError(f"Expected a non-empty value for `route_id` but received {route_id!r}")
        if not step_id:
            raise ValueError(f"Expected a non-empty value for `step_id` but received {step_id!r}")
        return self._put(
            f"/fleetify/routes/{route_id}/steps/{step_id}",
            body=maybe_transform(
                {
                    "arrival": arrival,
                    "position": position,
                    "address": address,
                    "completion_mode": completion_mode,
                    "document_template_id": document_template_id,
                    "duration": duration,
                    "geofence_config": geofence_config,
                    "location": location,
                    "meta": meta,
                    "type": type,
                },
                step_update_params.StepUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"key": key}, step_update_params.StepUpdateParams),
            ),
            cast_to=StepUpdateResponse,
        )

    def delete(
        self,
        step_id: str,
        *,
        route_id: str,
        key: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> StepDeleteResponse:
        """
        Delete a step

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not route_id:
            raise ValueError(f"Expected a non-empty value for `route_id` but received {route_id!r}")
        if not step_id:
            raise ValueError(f"Expected a non-empty value for `step_id` but received {step_id!r}")
        return self._delete(
            f"/fleetify/routes/{route_id}/steps/{step_id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"key": key}, step_delete_params.StepDeleteParams),
            ),
            cast_to=StepDeleteResponse,
        )

    def complete(
        self,
        step_id: str,
        *,
        route_id: str,
        key: str,
        document: DocumentSubmissionParam | NotGiven = NOT_GIVEN,
        mode: str | NotGiven = NOT_GIVEN,
        status: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> None:
        """
        Complete a route step with document submission, or update the document of a
        completed route step.

        When all steps are completed, the encapsulating route’s status will change to
        `completed` automatically.

        Either `Session Token` must be provided to authenticate the request.

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          document: A key-value map storing form submission data, where keys correspond to field
              labels and values can be of any type depend on the type of according document
              item.

          mode: Sets the status of the route step. Currently only `completed` is supported.

              Note: once marked `completed`, a step cannot transition to other statuses. You
              can only update the document afterwards.

          status: Sets the status of the route step. Currently only `completed` is supported.

              Note: once marked `completed`, a step cannot transition to other statuses. You
              can only update the document afterwards.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not route_id:
            raise ValueError(f"Expected a non-empty value for `route_id` but received {route_id!r}")
        if not step_id:
            raise ValueError(f"Expected a non-empty value for `step_id` but received {step_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._patch(
            f"/fleetify/routes/{route_id}/steps/{step_id}",
            body=maybe_transform(
                {
                    "document": document,
                    "mode": mode,
                    "status": status,
                },
                step_complete_params.StepCompleteParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"key": key}, step_complete_params.StepCompleteParams),
            ),
            cast_to=NoneType,
        )


class AsyncStepsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncStepsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncStepsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncStepsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#with_streaming_response
        """
        return AsyncStepsResourceWithStreamingResponse(self)

    async def create(
        self,
        route_id: str,
        *,
        key: str,
        arrival: int,
        location: Iterable[float],
        position: int,
        type: Literal["`start`", "`job`", "`pickup`", "`delivery`", "`break`", "`layover`", "`end`"],
        address: str | NotGiven = NOT_GIVEN,
        completion_mode: RouteStepCompletionMode | NotGiven = NOT_GIVEN,
        document_template_id: str | NotGiven = NOT_GIVEN,
        duration: int | NotGiven = NOT_GIVEN,
        geofence_config: RouteStepGeofenceConfigParam | NotGiven = NOT_GIVEN,
        meta: step_create_params.Meta | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> StepCreateResponse:
        """
        Insert a new step

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          arrival: Specify the scheduled arrival time of the driver, as an UNIX timestamp in
              seconds, at the step. Please note that:

              - Arrival time for each step should be equal to or greater than the previous
                step.
              - Past times can not be provided.
              - The time provided is used only for informative display on the driver app and
                it does not impact or get affected by the route generated.

          location: Specify the location coordinates where the steps should be performed in
              `[latitude, longitude]`.

          position: Indicates the index at which to insert the step, starting from 0 up to the total
              number of steps in the route.

          type: Specify the step type. It can belong to one of the following: `start`, `job` ,
              `pickup`, `delivery`, `end`. A `duration` is mandatory when the step type is
              either `layover` or a `break`.

          address: Specify the postal address for the step.

          completion_mode: Specify the mode of completion to be used for the step. Currently, following
              values are allowed:

              - `manual`: Steps must be marked as completed manually through the Driver App.
              - `geofence`: Steps are marked as completed automatically based on the entry
                conditions and geofence specified.
              - `geofence_manual_fallback`: Steps will be marked as completed automatically
                based on geofence and entry condition configurations but there will also be a
                provision for manually updating the status in case, geofence detection fails.

          document_template_id: Specify the ID of the document template to be used for collecting proof of
              completion for the step. If not specified, the document template specified at
              the route level will be used for the step. Use the
              [Documents API](https://docs.nextbillion.ai/docs/dispatches/documents-api) to
              create, read and manage the document templates.

              Please note that the document template ID can not be assigned to following step
              types - `start`, `end`, `break`, `layover`.

          duration: Specify the duration of the `layover` or `break` type steps, in seconds. Please
              note it is mandatory when step type is either "layover" or "break".

          geofence_config: Specify the configurations of the geofence which will be used to detect presence
              of the driver and complete the tasks automatically. Please note that this
              attribute is required when `completion_mode` is either "geofence" or
              "geofence_manual_fallback".

          meta: An object to specify any additional details about the task to be associated with
              the step in the response. The information provided here will be available on the
              Driver's app under step details. This attribute can be used to provide context
              about or instructions to the driver for performing the task

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not route_id:
            raise ValueError(f"Expected a non-empty value for `route_id` but received {route_id!r}")
        return await self._post(
            f"/fleetify/routes/{route_id}/steps",
            body=await async_maybe_transform(
                {
                    "arrival": arrival,
                    "location": location,
                    "position": position,
                    "type": type,
                    "address": address,
                    "completion_mode": completion_mode,
                    "document_template_id": document_template_id,
                    "duration": duration,
                    "geofence_config": geofence_config,
                    "meta": meta,
                },
                step_create_params.StepCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"key": key}, step_create_params.StepCreateParams),
            ),
            cast_to=StepCreateResponse,
        )

    async def update(
        self,
        step_id: str,
        *,
        route_id: str,
        key: str,
        arrival: int,
        position: int,
        address: str | NotGiven = NOT_GIVEN,
        completion_mode: RouteStepCompletionMode | NotGiven = NOT_GIVEN,
        document_template_id: str | NotGiven = NOT_GIVEN,
        duration: int | NotGiven = NOT_GIVEN,
        geofence_config: RouteStepGeofenceConfigParam | NotGiven = NOT_GIVEN,
        location: Iterable[float] | NotGiven = NOT_GIVEN,
        meta: step_update_params.Meta | NotGiven = NOT_GIVEN,
        type: Literal["`start`", "`job`", "`pickup`", "`delivery`", "`break`", "`layover`", "`end`"]
        | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> StepUpdateResponse:
        """
        Update a step

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          arrival: Specify the scheduled arrival time of the driver, as an UNIX timestamp in
              seconds, at the step. Please note that:

              - Arrival time for each step should be equal to or greater than the previous
                step.
              - Past times can not be provided.
              - The time provided is used only for informative display on the driver app and
                it does not impact or get affected by the route generated.

          position: Specify the new position of the step. Provide a position different from the
              current position of the step to update sequence in which the step get completed.

          address: Specify the postal address for the step.

          completion_mode: Specify the mode of completion to be used for the step. Currently, following
              values are allowed:

              - `manual`: Steps must be marked as completed manually through the Driver App.
              - `geofence`: Steps are marked as completed automatically based on the entry
                conditions and geofence specified.
              - `geofence_manual_fallback`: Steps will be marked as completed automatically
                based on geofence and entry condition configurations but there will also be a
                provision for manually updating the status in case, geofence detection fails.

          document_template_id: Update the ID of the document template to be used for collecting proof of
              completion for the step. If an empty string "" is provided, the current document
              template associated to the step will be removed.

          duration: Specify the duration of the `layover` or `break` type steps, in seconds. Please
              note it is mandatory when step type is either "layover" or "break".

          geofence_config: Specify the configurations of the geofence which will be used to detect presence
              of the driver and complete the tasks automatically. Please note that this
              attribute is required when `completion_mode` is either "geofence" or
              "geofence_manual_fallback".

          location: Specify the location coordinates where the steps should be performed in
              `[latitude, longitude]`.

          meta: An object to specify any additional details about the task to be associated with
              the step in the response. The information provided here will be available on the
              Driver's app under step details. This attribute can be used to provide context
              about or instructions to the driver for performing the task

          type: Specify the step type. It can belong to one of the following: `start`, `job` ,
              `pickup`, `delivery`, `end`. A `duration` is mandatory when the step type is
              either `layover` or a `break`.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not route_id:
            raise ValueError(f"Expected a non-empty value for `route_id` but received {route_id!r}")
        if not step_id:
            raise ValueError(f"Expected a non-empty value for `step_id` but received {step_id!r}")
        return await self._put(
            f"/fleetify/routes/{route_id}/steps/{step_id}",
            body=await async_maybe_transform(
                {
                    "arrival": arrival,
                    "position": position,
                    "address": address,
                    "completion_mode": completion_mode,
                    "document_template_id": document_template_id,
                    "duration": duration,
                    "geofence_config": geofence_config,
                    "location": location,
                    "meta": meta,
                    "type": type,
                },
                step_update_params.StepUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"key": key}, step_update_params.StepUpdateParams),
            ),
            cast_to=StepUpdateResponse,
        )

    async def delete(
        self,
        step_id: str,
        *,
        route_id: str,
        key: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> StepDeleteResponse:
        """
        Delete a step

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not route_id:
            raise ValueError(f"Expected a non-empty value for `route_id` but received {route_id!r}")
        if not step_id:
            raise ValueError(f"Expected a non-empty value for `step_id` but received {step_id!r}")
        return await self._delete(
            f"/fleetify/routes/{route_id}/steps/{step_id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"key": key}, step_delete_params.StepDeleteParams),
            ),
            cast_to=StepDeleteResponse,
        )

    async def complete(
        self,
        step_id: str,
        *,
        route_id: str,
        key: str,
        document: DocumentSubmissionParam | NotGiven = NOT_GIVEN,
        mode: str | NotGiven = NOT_GIVEN,
        status: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> None:
        """
        Complete a route step with document submission, or update the document of a
        completed route step.

        When all steps are completed, the encapsulating route’s status will change to
        `completed` automatically.

        Either `Session Token` must be provided to authenticate the request.

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          document: A key-value map storing form submission data, where keys correspond to field
              labels and values can be of any type depend on the type of according document
              item.

          mode: Sets the status of the route step. Currently only `completed` is supported.

              Note: once marked `completed`, a step cannot transition to other statuses. You
              can only update the document afterwards.

          status: Sets the status of the route step. Currently only `completed` is supported.

              Note: once marked `completed`, a step cannot transition to other statuses. You
              can only update the document afterwards.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not route_id:
            raise ValueError(f"Expected a non-empty value for `route_id` but received {route_id!r}")
        if not step_id:
            raise ValueError(f"Expected a non-empty value for `step_id` but received {step_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._patch(
            f"/fleetify/routes/{route_id}/steps/{step_id}",
            body=await async_maybe_transform(
                {
                    "document": document,
                    "mode": mode,
                    "status": status,
                },
                step_complete_params.StepCompleteParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"key": key}, step_complete_params.StepCompleteParams),
            ),
            cast_to=NoneType,
        )


class StepsResourceWithRawResponse:
    def __init__(self, steps: StepsResource) -> None:
        self._steps = steps

        self.create = to_raw_response_wrapper(
            steps.create,
        )
        self.update = to_raw_response_wrapper(
            steps.update,
        )
        self.delete = to_raw_response_wrapper(
            steps.delete,
        )
        self.complete = to_raw_response_wrapper(
            steps.complete,
        )


class AsyncStepsResourceWithRawResponse:
    def __init__(self, steps: AsyncStepsResource) -> None:
        self._steps = steps

        self.create = async_to_raw_response_wrapper(
            steps.create,
        )
        self.update = async_to_raw_response_wrapper(
            steps.update,
        )
        self.delete = async_to_raw_response_wrapper(
            steps.delete,
        )
        self.complete = async_to_raw_response_wrapper(
            steps.complete,
        )


class StepsResourceWithStreamingResponse:
    def __init__(self, steps: StepsResource) -> None:
        self._steps = steps

        self.create = to_streamed_response_wrapper(
            steps.create,
        )
        self.update = to_streamed_response_wrapper(
            steps.update,
        )
        self.delete = to_streamed_response_wrapper(
            steps.delete,
        )
        self.complete = to_streamed_response_wrapper(
            steps.complete,
        )


class AsyncStepsResourceWithStreamingResponse:
    def __init__(self, steps: AsyncStepsResource) -> None:
        self._steps = steps

        self.create = async_to_streamed_response_wrapper(
            steps.create,
        )
        self.update = async_to_streamed_response_wrapper(
            steps.update,
        )
        self.delete = async_to_streamed_response_wrapper(
            steps.delete,
        )
        self.complete = async_to_streamed_response_wrapper(
            steps.complete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .._models import BaseModel
from .rich_group_response import RichGroupResponse

__all__ = ["RestrictionListResponse", "Meta"]


class Meta(BaseModel):
    limit: Optional[int] = None
    """An integer value indicating the maximum number of items retrieved per "page".

    This is the same number as provided for the `limit` parameter in input.
    """

    offset: Optional[int] = None
    """
    An integer value indicating the number of items in the collection that were
    skipped to display the current response. Please note that the offset starts from
    zero.
    """

    total: Optional[int] = None
    """
    An integer value indicating the total number of items available in the data set.
    """


class RestrictionListResponse(BaseModel):
    data: Optional[List[RichGroupResponse]] = None
    """An array of objects containing the details of the restrictions returned.

    Each object represents one restriction.
    """

    meta: Optional[Meta] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["RichGroupResponse"]


class RichGroupResponse(BaseModel):
    id: Optional[int] = None
    """Returns the unique ID of the restriction.

    This ID can be used for update, delete, get operations on the restriction using
    the available API methods.
    """

    area: Optional[str] = None
    """Returns the area to which the restriction belongs to."""

    bbox: Optional[object] = None
    """Returns the details of the bounding box containing the restriction."""

    comment: Optional[str] = None
    """
    Returns the comments that were provided for the restriction at the time of
    creating or updating the request.
    """

    create_at: Optional[datetime] = None
    """The timestamp at which the restriction was created."""

    direction: Optional[Literal["forward", "backward", "both"]] = None
    """
    Returns the direction of travel on the segments to which the restriction
    applies.
    """

    end_time: Optional[float] = None
    """The time when the restriction ceases to be in-effect. It is a UNIX timestamp."""

    geofence: Optional[object] = None
    """
    Returns the list of coordinates representing the area that was used to apply the
    given restriction. The geofence returned is same as that provided while creating
    or updating the restriction.
    """

    highway: Optional[str] = None
    """Returns the highway information on which the restriction applies to.

    If no highway is impacted by the restriction, then this field is not present in
    the response.
    """

    mode: Optional[List[str]] = None
    """Returns an array denoting all the traveling modes the restriction applies on."""

    name: Optional[str] = None
    """Returns the name of the restriction.

    This value is same as that provided at the time of creating or updating the
    restriction.
    """

    repeat_on: Optional[str] = None
    """Returns the time periods during which this restriction active or repeats on.

    The time values follow a
    [given format](https://wiki.openstreetmap.org/wiki/Key:opening_hours).
    """

    restriction_type: Optional[Literal["closure", "maxspeed", "fixedspeed", "parking", "turn", "truck"]] = None
    """Returns the type of restriction.

    This is the same value as provided when creating or updating the restriction.
    """

    speed: Optional[float] = None
    """Returns the fixed speed of segments.

    This field is not present in the response if the restriction type is not
    `fixedspeed`
    """

    speed_limit: Optional[float] = None
    """Returns the maximum speed of segments.

    This field is not present in the response if the restriction type is not
    `maxspeed`
    """

    start_time: Optional[float] = None
    """The time when the restriction starts to be in-effect. It is a UNIX timestamp."""

    state: Optional[Literal["enabled", "disabled", "deleted"]] = None
    """Returns the state of the "restriction" itself - enabled, disabled or deleted.

    It does not denote if the restriction is actually in effect or not.
    """

    status: Optional[Literal["active", "inactive"]] = None
    """Returns the status of the restriction at the time of making the request i.e.

    whether the restriction is in force or not. It will have one of the following
    values: `active` or `inactive`.

    Please note that this field can not be directly influenced by the users. It will
    always be calculated using the `start_time`, `end_time` and `repeat_on`
    parameters.
    """

    update_at: Optional[datetime] = None
    """The timestamp at which the restriction was updated."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["Address"]


class Address(BaseModel):
    city: Optional[str] = None
    """The name of the primary locality of the place."""

    country_code: Optional[str] = FieldInfo(alias="countryCode", default=None)
    """A three-letter country code."""

    country_name: Optional[str] = FieldInfo(alias="countryName", default=None)
    """The localised country name."""

    county: Optional[str] = None
    """
    A division of a state; typically, a secondary-level administrative division of a
    country or equivalent.
    """

    district: Optional[str] = None
    """
    A division of city; typically an administrative unit within a larger city or a
    customary name of a city's neighborhood.
    """

    house_number: Optional[str] = FieldInfo(alias="houseNumber", default=None)
    """House number of the returned place, if available."""

    label: Optional[str] = None
    """
    Assembled address value built out of the address components according to the
    regional postal rules. These are the same rules for all endpoints. It may not
    include all the input terms.
    """

    postal_code: Optional[str] = FieldInfo(alias="postalCode", default=None)
    """
    An alphanumeric string included in a postal address to facilitate mail sorting,
    such as post code, postcode, or ZIP code.
    """

    state: Optional[str] = None
    """The state division of a country."""

    state_code: Optional[str] = FieldInfo(alias="stateCode", default=None)
    """A country specific state code or state name abbreviation.

    For example, in the United States it is the two letter state abbreviation: "CA"
    for California.
    """

    street: Optional[str] = None
    """Name of street of the returned place, if available."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["JsonRetrieveParams"]


class JsonRetrieveParams(TypedDict, total=False):
    destinations: Required[str]
    """\"destinations" are the ending coordinates of your route.

    Ensure that "destinations" are routable land locations. Multiple "destinations"
    should be separated by a pipe symbol "|".
    """

    key: Required[str]
    """
    A key is a unique identifier that is required to authenticate a request to the
    API.
    """

    origins: Required[str]
    """\"origins" are the starting point of your route.

    Ensure that "origins" are routable land locations. Multiple "origins" should be
    separated by a pipe symbol "|".
    """

    approaches: Literal["`unrestricted`", "`curb`"]
    """
    A semicolon-separated list indicating the side of the road from which the route
    will approach "destinations". When set to "unrestricted" a route can arrive at a
    destination from either side of the road. When set to "curb" the route will
    arrive at a destination on the driving side of the region. Please note the
    number of values provided must be equal to the number of "destinations".
    However, you can skip a coordinate and show its position in the list with the
    ";" separator. The values provided for the "approaches" parameter are effective
    for the "destinations" value at the same index. Example: "curb;;curb" will apply
    curbside restriction on the "destinations" points provided at the first and
    third index.
    """

    avoid: Literal["toll", "ferry", "highway", "none"]
    """Setting this will ensure the route avoids ferries, tolls, highways or nothing.

    Multiple values should be separated by a pipe (|). If "none" is provided along
    with other values, an error is returned as a valid route is not feasible. Please
    note that when this parameter is not provided in the input, ferries are set to
    be avoided by default. When this parameter is provided, only the mentioned
    objects are avoided.
    """

    bearings: str
    """
    Limits the search to segments with given bearing in degrees towards true north
    in clockwise direction. Each "bearing" should be in the format of
    "degree,range", where the "degree" should be a value between \\[[0, 360\\]] and
    "range" should be a value between \\[[0, 180\\]]. Please note that the number of
    "bearings" should be equal to the sum of the number of points in "origins" and
    "destinations". If a route can approach a destination from any direction, the
    bearing for that point can be specified as "0,180".
    """

    mode: Literal["`car`", "`truck`"]
    """
    Set which driving mode the service should use to determine the "distance" and
    "duration" values. For example, if you use "car", the API will return the
    duration and distance of a route that a car can take. Using "truck" will return
    the same for a route a truck can use, taking into account appropriate truck
    routing restrictions.

    When "mode=truck", following are the default dimensions that are used:

    \\-- truck_height = 214 centimeters

    \\-- truck_width = 183 centimeters

    \\-- truck_length = 519 centimeters

    \\-- truck_weight = 5000 kg

    Please use the Distance Matrix Flexible version if you want to use custom truck
    dimensions.

    Note: Only the "car" profile is enabled by default. Please note that customized
    profiles (including "truck") might not be available for all regions. Please
    contact your [NextBillion.ai](http://NextBillion.ai) account manager, sales
    representative or reach out at
    [support@nextbillion.ai](mailto:support@nextbillion.ai) in case you need
    additional profiles.
    """

    route_failed_prompt: bool
    """
    A prompt to modify the response in case no feasible route is available for a
    given pair of origin and destination. When set to "true", a value of "-1" is
    returned for those pairs in which:

    \\-- Either origin or the destination can not be snapped to a nearest road. Please
    note that if all the origins and destinations in a request can't be snapped to
    their nearest roads, a 4xx error is returned instead, as the entire request
    failed.

    \\-- Both origin and destination can be snapped to the nearest road, but the
    service can't find a valid route between them. However, a value of "0" is
    returned if both the origin and destination are snapped to the same location.

    "false" is the default value. In this case, a "0" value is returned for all the
    above cases. A 4xx error is returned, in this case as well, when all origins and
    destinations in the request can't be snapped to their nearest road.
    """
